@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,14 +18,15 @@
 */
 
 #include "bodisplaymanager.h"
-#include "bodisplaymanager.moc"
 
 #include "defines.h"
 #include "bosonbigdisplaybase.h"
 #include "bosonbigdisplayinputbase.h"
 #include "bosonconfig.h"
 #include "boselection.h"
+#include "bosoncanvas.h"
 #include "bodebug.h"
+#include "boaction.h"
 
 #include <klocale.h>
 
@@ -34,6 +35,12 @@
 #include <qvbox.h>
 #include <qintdict.h>
 #include <qdom.h>
+#include <qfile.h>
+#include <qfileinfo.h>
+#include <qpixmap.h>
+
+#include "bodisplaymanager.moc"
+
 
 class BoBox : public QWidget
 {
@@ -104,19 +111,20 @@ class BoDisplayManager::BoDisplayManagerPrivate
 public:
 	BoDisplayManagerPrivate()
 	{
-		mActiveDisplay = 0;
-
 		mLayout = 0;
+
+		mActiveDisplay = 0;
 	}
 
-	QPtrList<BosonBigDisplayBase> mDisplayList;
-	QPtrList<BoBox> mBoxList;
-
-	BosonBigDisplayBase* mActiveDisplay;
-
 	QVBoxLayout* mLayout;
 
+	QPtrList<BosonBigDisplayBase> mDisplayList;
+	QPtrList<BoBox> mBoxList;
+	BosonBigDisplayBase* mActiveDisplay;
+
 	QIntDict<BoSelection> mSelectionGroups;
+
+	bool mGrabMovie;
 };
 
 BoDisplayManager::BoDisplayManager(QWidget* parent) : QWidget(parent, "bosondisplaymanager")
@@ -124,6 +132,7 @@ BoDisplayManager::BoDisplayManager(QWidget* parent) : QWidget(parent, "bosondisp
  d = new BoDisplayManagerPrivate;
  d->mDisplayList.setAutoDelete(true);
  d->mBoxList.setAutoDelete(true);
+ d->mGrabMovie = false;
 
  d->mSelectionGroups.setAutoDelete(true);
  for (int i = 0; i < 10; i++) {
@@ -332,6 +341,11 @@ void BoDisplayManager::setCanvas(BosonCanvas* c)
 	it.current()->setCanvas(c);
 	++it;
  }
+ QIntDictIterator<BoSelection> selectIt(d->mSelectionGroups);
+ for (; selectIt.current(); ++selectIt) {
+	connect(c, SIGNAL(signalRemovedItem(BosonItem*)),
+			selectIt.current(), SLOT(slotRemoveItem(BosonItem*)));
+ }
 }
 
 void BoDisplayManager::quitGame()
@@ -342,6 +356,9 @@ void BoDisplayManager::quitGame()
 	it.current()->quitGame();
 	++it;
  }
+ for (int i = 0; i < 10; i++) {
+	slotClearGroup(i);
+ }
 }
 
 BoBox* BoDisplayManager::findBox(BosonBigDisplayBase* b) const
@@ -392,13 +409,51 @@ void BoDisplayManager::slotScroll(int dir)
  }
 }
 
+void BoDisplayManager::slotRotateLeft()
+{
+ BosonBigDisplayBase* active = activeDisplay();
+ if (!active) {
+	return;
+ }
+ active->rotateLeft();
+}
+
+void BoDisplayManager::slotRotateRight()
+{
+ BosonBigDisplayBase* active = activeDisplay();
+ if (!active) {
+	return;
+ }
+ active->rotateRight();
+}
+
+void BoDisplayManager::slotZoomIn()
+{
+ BosonBigDisplayBase* active = activeDisplay();
+ if (!active) {
+	return;
+ }
+ active->zoomIn();
+}
+
+void BoDisplayManager::slotZoomOut()
+{
+ BosonBigDisplayBase* active = activeDisplay();
+ if (!active) {
+	return;
+ }
+ active->zoomOut();
+}
+
 void BoDisplayManager::slotAdvance(unsigned int, bool)
 {
  QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
  while (it.current()) {
 	it.current()->setParticlesDirty(true);
+	it.current()->advanceCamera();
 	++it;
  }
+ grabMovieFrame();
 }
 
 void BoDisplayManager::slotUpdateIntervalChanged(unsigned int ms)
@@ -471,14 +526,6 @@ void BoDisplayManager::slotClearGroup(int number)
  d->mSelectionGroups[number]->clear();
 }
 
-void BoDisplayManager::slotUnitAction(int action)
-{
- BO_CHECK_NULL_RET(activeDisplay());
- BO_CHECK_NULL_RET(activeDisplay()->displayInput());
-
- activeDisplay()->displayInput()->unitAction(action);
-}
-
 void BoDisplayManager::slotPlaceUnit(unsigned long int unitType, Player* owner)
 {
  BO_CHECK_NULL_RET(activeDisplay());
@@ -487,12 +534,12 @@ void BoDisplayManager::slotPlaceUnit(unsigned long int unitType, Player* owner)
  activeDisplay()->displayInput()->placeUnit(unitType, owner);
 }
 
-void BoDisplayManager::slotPlaceCell(int tile)
+void BoDisplayManager::slotPlaceGround(unsigned int textureCount, unsigned char* alpha)
 {
  BO_CHECK_NULL_RET(activeDisplay());
  BO_CHECK_NULL_RET(activeDisplay()->displayInput());
 
- activeDisplay()->displayInput()->placeCell(tile);
+ activeDisplay()->displayInput()->placeGround(textureCount, alpha);
 }
 
 void BoDisplayManager::slotMoveActiveSelection(int x, int y)
@@ -564,60 +611,6 @@ void BoDisplayManager::slotUnitChanged(Unit* unit)
  }
 }
 
-void BoDisplayManager::slotSetDebugMapCoordinates(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugMapCoordinates(debug);
-	++it;
- }
-}
-
-void BoDisplayManager::slotSetDebugShowCellGrid(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugShowCellGrid(debug);
-	++it;
- }
-}
-
-void BoDisplayManager::slotSetDebugMatrices(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugMatrices(debug);
-	++it;
- }
-}
-
-void BoDisplayManager::slotSetDebugItemWorks(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugItemWorks(debug);
-	++it;
- }
-}
-
-void BoDisplayManager::slotSetDebugCamera(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugCamera(debug);
-	++it;
- }
-}
-
-void BoDisplayManager::slotSetDebugRenderCounts(bool debug)
-{
- QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
- while (it.current()) {
-	it.current()->setDebugRenderCounts(debug);
-	++it;
- }
-}
-
 void BoDisplayManager::unlockAction()
 {
  QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
@@ -742,4 +735,89 @@ void BoDisplayManager::slotChangeViewport(BosonBigDisplayBase* display, const QP
  emit signalChangeActiveViewport(topLeft, topRight, bottomLeft, bottomRight);
 }
 
+void BoDisplayManager::slotAction(const BoSpecificAction& action)
+{
+ BO_CHECK_NULL_RET(activeDisplay());
+ BO_CHECK_NULL_RET(activeDisplay()->displayInput());
+
+ activeDisplay()->displayInput()->action(action);
+}
+
+QPtrList<BosonBigDisplayBase>* BoDisplayManager::displayList()
+{
+ return &d->mDisplayList;
+}
+
+void BoDisplayManager::slotChangeGroundRenderer(int g)
+{
+ QPtrListIterator<BosonBigDisplayBase> it(d->mDisplayList);
+ while (it.current()) {
+	it.current()->changeGroundRenderer(g);
+	++it;
+ }
+}
+
+void BoDisplayManager::slotSetGrabMovie(bool grab)
+{
+ BO_CHECK_NULL_RET(activeDisplay());
+
+ d->mGrabMovie = grab;
+}
+
+void BoDisplayManager::grabMovieFrame()
+{
+ if (!d->mActiveDisplay) {
+	return;
+ }
+ if (!d->mGrabMovie) {
+	return;
+ }
+ QByteArray shot = d->mActiveDisplay->grabMovieFrame();
+
+ if (shot.size() == 0) {
+	return;
+ }
+
+ // Save frame
+ static int frame = -1;
+ QString file;
+ if (frame == -1) {
+	int i;
+	for (i = 0; i <= 10000; i++) {
+		file.sprintf("%s-%04d.%s", "boson-movie", i, "jpg");
+		if (!QFile::exists(file)) {
+			frame = i;
+			break;
+		}
+	}
+	if (i == 10000) {
+		boWarning() << k_funcinfo << "Can't find free filename???" << endl;
+		frame = 50000;
+	}
+ }
+ file.sprintf("%s-%04d.%s", "boson-movie", frame++, "jpg");
+ file = QFileInfo(file).absFilePath();
+
+ //boDebug() << k_funcinfo << "Saving movie frame to " << file << endl;
+ bool ok = QPixmap(shot).save(file, "JPEG", 90);
+ if (!ok) {
+	boError() << k_funcinfo << "Error saving screenshot to " << file << endl;
+	return;
+ }
+ boDebug() << k_funcinfo << "Movie frame saved to file " << file << endl;
+
+#if 0
+ static QValueList<QByteArray> allMovieFrames;
+ allMovieFrames.append(shot);
+
+
+ // TODO: use a shortcut for this. do not do this after a certain number of
+ // frames, but when a key was pressed.
+ if (allMovieFrames.count() == 10) {
+	boDebug() << k_funcinfo << "generating " << allMovieFrames.count() << " frames" << endl;
+	d->mActiveDisplay->generateMovieFrames(allMovieFrames, "./11/");
+	allMovieFrames.clear();
+ }
+#endif
+}
 
