@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@ class BosonPlayField;
 class QDomElement;
 class QDomDocument;
 class QDataStream;
+class QTextStream;
+class BosonSaveLoad;
+class BosonStarting;
 
 #define boGame Boson::boson()
 
@@ -46,63 +49,94 @@ public:
 	enum PropertyIds {
 		IdGameSpeed = 10000, // dont wanna #include <kgameproperty.h> - better: KGamePropertyBase::IdUser+...
 		IdGamePaused = 10001,
-		IdNextUnitId = 10005,
 		IdAdvanceCount = 10010,
-		IdAdvanceFlag = 10011
-	};
-
-	/**
-	 * Describes current status when loading saved game
-	 * This is mostly used for error checking
-	 *
-	 * Possible values:
-	 * @li NotLoaded - Game is not yet loaded (loading method isn't called yet)
-	 * @li LoadingInProgress - Loading is in progress
-	 * @li LoadingCompleted - Loading is completed (successfully)
-	 * @li BSGFileError - Error in BSGFile. Most likely it wasn't Boson savegame
-	 * @li InvalidXML - Error in one of XML files.
-	 * @li InvalidFileFormat - File format was invalid (error)
-	 * @li InvalidCookie - Cookie in the file header was invalid (error)
-	 * @li InvalidVersion - SaveGame version was invalid. Probably the game was saved with too old version of Boson (error)
-	 * @li KGameError - Error while loading KGame stuff
-	 **/
-	enum LoadingStatus {
-		NotLoaded = 1,
-		LoadingInProgress,
-		LoadingCompleted,
-		BSGFileError,
-		InvalidXML,
-		InvalidFileFormat,
-		InvalidCookie,
-		InvalidVersion,
-		KGameError
+		IdAdvanceFlag = 10011,
+		IdAdvanceCallsCount = 10020
 	};
 
+protected:
 	Boson(QObject* parent);
-	Boson(QObject* parent, const QString& fileName);
 	~Boson();
 
-	void setCanvas(BosonCanvas*);
+public:
+	/**
+	 * @return The global Boson object
+	 **/
+	static Boson* boson() { return mBoson; }
+
+	/**
+	 * Initialize the global Boson object. See also @ref boson.
+	 **/
+	static void initBoson();
+
+	/**
+	 * Clean up the global Boson object.
+	 **/
+	static void deleteBoson();
+
+public:
+	/**
+	 * Create a new @ref BosonCanvas object. The object will be deleted when
+	 * this Boson object is destroyed, see @ref deleteBoson.
+	 **/
+	void createCanvas();
+
+	/**
+	 * @return The @ref BosonCanvas object. See also @ref createCanvas.
+	 *
+	 * Note That I do <em>not</em> want to see a non-const pointer returned
+	 * in this class! Boson is a static object that is available
+	 * _everywhere_ in this program and I do not want to allow access to the
+	 * @ref BosonCanvas object all over this application.
+	 *
+	 * This pointer is meant to be used for e.g. statistic information (how
+	 * many units are on the map, how many particle systems, ...)
+	 **/
+	const BosonCanvas* canvas() const;
+	BosonCanvas* canvasNonConst() const; // FIXME: remove!
+
 	void setPlayField(BosonPlayField*);
 	BosonPlayField* playField() const;
 
 	void setLocalPlayer(Player*);
 	Player* localPlayer() const;
 
+	/**
+	 * Set the object that will get used to start the game. Some starting
+	 * relevant network messages could get forwarded directly to this object
+	 * and we will store some data from network messages there.
+	 *
+	 * Note that a NULL @p starting parameter will unset the object and
+	 * therefore disable game starting. This can be done once the game has
+	 * been started, in order to avoid starting it twice.
+	 **/
+	void setStartingObject(BosonStarting* starting);
+
+	/**
+	 * Initialize a @ref BosonSaveLoad object with the relevant data.
+	 **/
+	void initSaveLoad(BosonSaveLoad*);
+
 	void quitGame();
 	void startGame();
 
 	int gameSpeed() const;
 	bool gamePaused() const;
-	bool isServer() const;
 
-	virtual KPlayer* createPlayer(int rtti, int io, bool isVirtual);
 	void removeAllPlayers();
 
-	Unit* loadUnit(unsigned long int unitType, Player* owner);
-
 	QValueList<QColor> availableTeamColors() const;
 
+	/**
+	 * Kill the @p player, i.e. remove all it's units (if any) from the
+	 * game. Once this was called the player is not able to do anything
+	 * anymore.
+	 *
+	 * Note that the @p player will remain in the game, but can watch only
+	 * after this point. The player is not removed from the network.
+	 **/
+	void killPlayer(Player* player);
+
 	/**
 	 * @param unitId The unit to search for
 	 * @param searchIn The player to search the unit in. 0 for all players
@@ -120,16 +154,27 @@ public:
 	 **/
 	bool buildProducedUnit(ProductionPlugin* factory, unsigned long int unitType, int x, int y);
 
-	/**
-	 * Behaves slightly similar to @ref slotSendAddUnit but this function
-	 * takes an xml document and you can add several units at once.
-	 **/
-	void sendAddUnits(const QString& xmlDocument, Player* owner);
-
 	virtual void networkTransmission(QDataStream&, int msgid, Q_UINT32 receiver, Q_UINT32 sender, Q_UINT32 clientID);
+
+	/**
+	 * Lock message delivery until @ref unlock is called.
+	 *
+	 * Messages are not lost, but just delayed.
+	 **/
 	virtual void lock();
+
+	/**
+	 * Unlock message delivery (see @ref lock) and deliver all delayed
+	 * messages (see @ref slotProcessDelayed)
+	 **/
 	virtual void unlock();
 
+	bool isLocked() const;
+
+	/**
+	 * Set the game/editor mode. Use TRUE here for normal game mode, FALSE
+	 * for editor mode.
+	 **/
 	void setGameMode(bool isGame = true) { mGameMode = isGame; }
 	
 	/**
@@ -141,14 +186,16 @@ public:
 		return mGameMode;
 	}
 
-	bool loadFromFile(const QString& file);
 	bool saveToFile(const QString& file);
+	bool savePlayFieldToFile(const QString& file);
 
 	virtual bool savegame(QDataStream& stream, bool network, bool saveplayers = true);
-	virtual bool load(QDataStream& stream, bool reset = true);
 	virtual bool loadgame(QDataStream& stream, bool network, bool reset);
 
-	LoadingStatus loadingStatus() const;
+	/**
+	 * @return See @ref BosonSaveLoad::LoadingStatus
+	 **/
+	int loadingStatus() const;
 
 	/**
 	 * See @ref Unit::advanceFunction2 for more info about this.
@@ -162,8 +209,6 @@ public:
 	 **/
 	void toggleAdvanceFlag();
 
-	virtual bool save(QDataStream& stream, bool savePlayers = true);
-
 	/**
 	 * @return The number of delayed messages. When a certain number of
 	 * messages has been delayed you are in serious trouble, as there will
@@ -172,24 +217,69 @@ public:
 	unsigned int delayedMessageCount() const;
 
 	/**
-	 * @return The global Boson object
+	 * The number of delayed advance messages is really important in a game.
+	 * In theory we should have correct timing, which ensures, that no
+	 * advance message ever gets delayed. in practice, this just isn't the
+	 * case.
+	 *
+	 * Once an advance message is delayed, everything in the game lags by a
+	 * certain amount of time (the interval in which advance messages are
+	 * sent). You should do everything to process all delayed messages asap,
+	 * as multiple delayed advance messages can be noticed by the user (he
+	 * clicks, to make a unit move, but the unit moves after some seconds
+	 * only).
+	 *
+	 * Usually the best way to get rid of delayed advance messages is to
+	 * drop frames. Frame rendering takes a huge amount of time!
+	 * @return The number of advance messages in @ref delayedMessageCount.
 	 **/
-	static Boson* boson() { return mBoson; }
+	unsigned int delayedAdvanceMessageCount() const;
 
 	/**
-	 * Initialize the global Boson object. See also @ref boson.
+	 * @return Number of advance calls that have been made in this game (aka
+	 * cycles)
 	 **/
-	static void initBoson();
+	unsigned int advanceCallsCount() const;
+
+	void writeGameLog(QTextStream& log);
+	void saveGameLogs(const QString& prefix);
+
 
 	/**
-	 * Clean up the global Boson object.
+	 * Init the fog of war. This must not be called anymore, once a game is
+	 * started, therefore it is allowed from a @ref BosonStarting object
+	 * only.
 	 **/
-	static void deleteBoson();
+	void initFogOfWar(BosonStarting*);
 
 	/**
-	 * @return Latest Boson savegame format version
+	 * Start the scenario. This must not be called anymore, once a game is
+	 * started, therefore it is allowed from a @ref BosonStarting object
+	 * only.
 	 **/
-	unsigned long int latestSavegameVersion();
+	void startScenario(BosonStarting*);
+
+
+public: // small KGame extenstions for boson
+	/**
+	 * Used internally by @ref KGame. Simply create and return a new @ref
+	 * Player object.
+	 **/
+	virtual KPlayer* createPlayer(int rtti, int io, bool isVirtual);
+
+	/**
+	 * Simply calls @ref loadgame. This is an old function, that resides
+	 * here due to an old KGame bug.
+	 **/
+	virtual bool load(QDataStream& stream, bool reset = true);
+
+	/**
+	 * Dummy implementation that simply calls @ref KGame::save or @ref
+	 * savegame, depending on the @ref KGame version. @ref KGame from KDE
+	 * 3.0 didn't have @ref savegame, so we need to emulate it on our own
+	 * here.
+	 **/
+	virtual bool save(QDataStream& stream, bool savePlayers = true);
 
 	/**
 	 * @return The port that is used for network games. The port we listen
@@ -197,21 +287,42 @@ public:
 	 * another client.
 	 **/
 	Q_UINT16 bosonPort();
+	bool isServer() const;
+
+	/**
+	 * @return @ref KGameNetwork::hotName, if present (was added in KDE
+	 * 3.2). If it is not present, we use an evil hack that should have the
+	 * same result.
+	 **/
+	QString bosonHostName();
+
+	/**
+	 * Workaround for a KGame bug in KDE < 3.2. This just calls @ref
+	 * KGame::addPlayer directly in KDE >= 3.2, otherwise it will first
+	 * assign a correct ID to the player and then call it.
+	 **/
+	void bosonAddPlayer(KPlayer* player);
+
+	/**
+	 * Dummy systemAddPlayer function. This is meant to grant access to
+	 * systemAddPlayer to @ref BosonSaveLoad, without making the class a
+	 * friend of Boson (i.e. granting access to whole Boson).
+	 *
+	 * A method BosonSaveLoad::systemAddPlayer that is friend of Boson would
+	 * have been better (a lot!) but requires us to include bosonsaveload.h
+	 * here.
+	 **/
+	void systemAddPlayer_(BosonSaveLoad* b, KPlayer* player)
+	{
+		if (b) {
+			systemAddPlayer(player);
+		}
+	}
 
 public slots:
 	void slotSetGameSpeed(int speed);
 	void slotTogglePause();
 
-	/**
-	 * Doesn't actually add a unit to the game but sends a message to the
-	 * network. The actual adding is done in @ref slotNetworkData
-	 * @param unitType The type of the unit (see @ref UnitProperties::typeId) to be added
-	 * @param x The x-coordinate (on the canvas) of the unit
-	 * @param y The y-coordinate (on the canvas) of the unit
-	 * @param owner The owner of the new unit.
-	 **/
-	void slotSendAddUnit(unsigned long int unitType, int x, int y, Player* owner);
-
 	void slotAdvanceComputerPlayers(unsigned int advanceCount, bool advanceFlag);
 
 	/**
@@ -252,21 +363,6 @@ signals:
 	 * Now all clients should start loading tiles etc.
 	 **/
 	void signalStartNewGame();
-	
-	/**
-	 * Start a scenario. This should be done after loading map and scenario
-	 * (a scenario is loaded first, <em>then</em> started). It is
-	 * implemented using @ref KGame::sendMessage as the map must be loaded
-	 * this way as well.
-	 **/
-	void signalStartScenario();
-
-	/**
-	 * @param unit The unit to be added
-	 * @param x x-coordinate of the unit on the canvas
-	 * @param y y-coordinate of the unit on the canvas
-	 **/
-	void signalAddUnit(Unit* unit, int x, int y);
 
 	/**
 	 * Order the canvas to call @ref BosonCanvas::slotAdvance
@@ -278,8 +374,6 @@ signals:
 	 **/
 	void signalAdvance(unsigned int advanceCount, bool advanceFlag);
 
-	void signalInitMap(const QByteArray&);
-
 	void signalLoadPlayerData(Player* player);
 	void signalLoadingPlayersCount(int count);
 	void signalLoadingPlayer(int current);
@@ -332,16 +426,12 @@ signals:
 	void signalUpdateProductionOptions();
 
 	/**
-	 * Tell the map to change a cell at @p x, @p y to @p groundType and @p
-	 * version. This must not be connected to a slot in game mode (but it
-	 * also must not be emitted then).
+	 * Tell the map to change @ref BosonMap::texMap at coordinates @p x, @p
+	 * y.
 	 *
-	 * Note that we you can use this to update the minimap as well, but be
-	 * aware that when the minimap slot gets called the map itself might not
-	 * have been updated yet, since we can't safely predict the order in which
-	 * slots get called.
+	 * This is for editor use only!
 	 **/
-	void signalChangeCell(int x, int y, int groundType, unsigned char version );
+	void signalChangeTexMap(int x, int y, unsigned int textureCount, unsigned int* textures, unsigned char* alpha);
 
 	void signalAddChatSystemMessage(const QString& fromName, const QString& text);
 
@@ -350,34 +440,20 @@ signals:
 	void signalLoadExternalStuffFromXML(const QDomElement& root);
 	void signalSaveExternalStuffAsXML(QDomElement& root);
 
+	/**
+	 * Emitted when a player is killed, i.e. has lost. The player
+	 * still remains in the network (i.e. the game), he just cannot
+	 * play anymore.
+	 * This is emitted mainly by @ref killPlayer.
+	 *
+	 * The player is <em>not</em> removed from network and can take part in
+	 * the next game for example. When the player is removed from the
+	 * network @ref signalPlayerLeftGame is emitted.
+	 **/
+	void signalPlayerKilled(Player*);
+
 protected:
 	virtual bool playerInput(QDataStream& stream, KPlayer* player);
-	Unit* createUnit(unsigned long int unitType, Player* owner);
-
-	unsigned long int nextUnitId();
-
-	/**
-	 * Create a new unit. No resources of the player are reduced, the unit
-	 * is created immediately.
-	 **/
-	Unit* addUnit(unsigned long int unitType, Player* owner, int x, int y);
-
-	/**
-	 * Create a unit from node. This behaves similar to the above function,
-	 * but you can specify <em>every</em> property in the node, not just
-	 * type and position. 
-	 *
-	 * This is used by @ref BosonScenario.
-	 **/
-	Unit* addUnit(QDomElement& node, Player* owner);
-
-	QString saveKGameAsXML();
-	QString savePlayersAsXML();
-	QString saveCanvasAsXML();
-	QString saveExternalAsXML();
-	bool loadKGameFromXML(const QString&);
-	bool loadCanvasFromXML(const QString&);
-	bool loadExternalFromXML(const QString&);
 
 	/**
 	 * Load the XML file in @p xml into @p doc and display an error message
@@ -386,6 +462,12 @@ protected:
 	 **/
 	bool loadXMLDoc(QDomDocument* doc, const QString& xml);
 
+	/**
+	 * Create a game log (see @ref writeGameLog) and store it for later use (see
+	 * @ref saveGameLogs).
+	 **/
+	void makeGameLog();
+
 protected slots:
 	/**
 	 * A network message arrived. Most game logic stuff is done here as
@@ -414,6 +496,9 @@ protected slots:
 
 	void slotDebugOutput(const QString& area, const char* data, int level);
 
+private:
+	friend class BoAdvance;
+
 private:
 	class BosonPrivate;
 	BosonPrivate* d;
@@ -422,4 +507,37 @@ private:
 	static Boson* mBoson;
 };
 
+class QSocket;
+class KMessageSocket;
+/**
+ * Hack for a KMessageSocket bug in KDE < 3.2
+ **/
+class BoMyKMessageSocket : public QObject
+{
+	Q_OBJECT
+public:
+	BoMyKMessageSocket() : QObject(0)
+	{
+		mSocketSocket = 0;
+		mSocket = 0;
+		mNextBlockLength = 0;
+		mAwaitingHeader = 0;
+	}
+
+	void setSocket(KMessageSocket* s, QSocket* sock);
+
+public slots:
+	void slotProcessNewData();
+
+signals:
+	void signalReceived(const QByteArray&);
+
+private:
+	QSocket* mSocketSocket;
+	KMessageSocket* mSocket;
+	bool mAwaitingHeader;
+	Q_UINT32 mNextBlockLength;
+};
+
 #endif
+
