@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ public:
 	enum BosonMessages {
 	// messages for the newgame dialog/widget:
 	// (a few might be obsolete or moved to another place)
-		InitMap = 0,
+//		InitMap = 0,
 		ChangeSpecies = 1,
 		ChangePlayField = 2,
 		ChangeTeamColor = 4,
@@ -37,13 +37,13 @@ public:
 		IdStartGameClicked = 7,
 
 	// once a newgame is started:
-		IdInitFogOfWar = 10,
-		IdStartScenario = 11,
-		AddUnitsXML = 12, // add units from an xml (BosonScenario) file/stream
+//		IdInitFogOfWar = 10,
+//		IdStartScenario = 11,
+//		AddUnitsXML = 12, // add units from an xml (BosonScenario) file/stream
 		IdGameIsStarted = 25, // the game is started as soon as this is received
 
 	// usually withing a game
-		AddUnit = 30, // whenever a unit is added
+//		AddUnit = 30, // whenever a unit is added
 		ChangeMap = 31, // editor only (but comparable to AddUnit)
 
 	// debug/cheat messages
@@ -68,10 +68,16 @@ public:
 		MoveStop = 108,  // Stop unit(s) from moving or attacking
 		MoveFollow = 109,  // Follow another unit
 
+		MoveLayMine = 120,
+		MoveDropBomb = 121,
+
+		MoveTeleport = 150,  // Immediately move unit (set it's position)
+		MoveRotate = 151,  // Set unit's rotation
+
 	// Player Input in Editor mode
 		MoveEditor = 200, // all editor moves/inputs are prefixed with this
 		MovePlaceUnit = 201,
-		MovePlaceCell = 202,
+		MoveChangeTexMap = 202,
 		MoveChangeHeight = 203,
 
 
