@@ -56,14 +56,19 @@ public:
 		SavePlayFieldToXML = 24,
 		SaveGameToXMLWriteFile = 25,
 		SaveCanvasToXML = 26,
-		SaveExternalToXML = 27
+		SaveExternalToXML = 27,
+		PreLoadPlayFields = 40,
+		LoadPlayField = 41,
+		FindPath = 100
 	};
 	BosonProfiling();
 	BosonProfiling(const BosonProfiling& profiling);
 	~BosonProfiling();
 
-	static void initProfiling();
-	static BosonProfiling* bosonProfiling() { return mProfiling; }
+	/**
+	 * @return BoGlobal::boGlobal()->bosonProfiling();
+	 **/
+	static BosonProfiling* bosonProfiling();
 
 	/**
 	 * Change the OpenGL update interval. This value can be useful when
@@ -108,16 +113,44 @@ public:
 	 **/
 	void start(int event);
 
+	/**
+	 * @return The elapsed time since calling @ref start with this event
+	 * number (in usec). Undefined if @ref start has not yet been called
+	 * with this event number.
+	 **/
+	long int elapsed(int event) const;
+
 	/**
 	 * Stop the event timer and append the resulting time to the list. If
 	 * the list contains more than MAX_ENTRIES the first item is removed.
 	 * See also @ref start
+	 *
+	 * @param appendToList If TRUE (default) the resulting time will be
+	 * appended to the list for this event, and can be analyzed using @ref
+	 * BosonProfilingDialog. Otherwise the elapsed time is returned only.
+	 *
+	 * @return See @ref elapsed (note: usec!)
 	 **/
-	void stop(int event);
+	long int stop(int event, bool appendToList = true);
 
 	void loadUnit();
 	void loadUnitDone(unsigned long int typeId);
 
+	/**
+	 * Starts benchmark
+	 * During benchmark, all profiling info about advance calls and rendered
+	 * frames is logged and some statistics are printed out when you call
+	 * @ref endBenchmark()
+	 **/
+	void startBenchmark();
+	/**
+	 * Ends benchmark and prints out some useful info about logged things.
+	 *
+	 * @param name Optional name of the benchmark. If given it's printed out, so
+	 * you can later indentify your benchmark
+	 **/
+	void endBenchmark(const QString& name);
+
 	// WARNING: do !NOT! call render*() or advance*() before you called
 	// render(true)/advance(true) or after render(false)/advance(false) !
 	// that would crash! (no NULL check in favor of performance)
@@ -180,8 +213,6 @@ private:
 private:
 	BosonProfilingPrivate* d;
 	friend class BosonProfilingDialog;
-
-	static BosonProfiling* mProfiling;
 };
 
 unsigned long int compareTimes(const struct timeval& t1, const struct timeval& t2);
@@ -242,11 +273,20 @@ public:
 	/**
 	 * This stops profiling the event immediately. The destructor will do
 	 * nothing when you call this.
+	 * @return See @ref BosonProfiling::stop
 	 **/
-	void stop()
+	long int stop()
 	{
-		boProfiling->stop(mEvent);
 		mEventStopped = true;
+		return boProfiling->stop(mEvent);
+	}
+
+	/**
+	 * @return See @ref BosonProfiling::elapsed
+	 **/
+	long int elapsed() const
+	{
+		return boProfiling->elapsed(mEvent);
 	}
 
 private:
