@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,14 @@
 #include "bosoncommandframebase.h"
 #include "bosoncommandframebase.moc"
 
+#include "bosoncommandframe.h"
+#include "editorcommandframe.h"
+
 #include "bosonorderwidget.h"
 #include "bosonunitview.h"
 #include "../unit.h"
 #include "../player.h"
+#include "../playerio.h"
 #include "../speciestheme.h"
 #include "../boselection.h"
 #include "../unitplugins.h"
@@ -66,7 +70,7 @@ BoUnitDisplayBase::BoUnitDisplayBase(BosonCommandFrameBase* frame, QWidget* pare
  mCommandFrame = frame;
  mUpdateTimer = false;
  if (!frame) {
-	boError() << k_funcinfo << "NULL cmdFrame" << endl;
+	boError(220) << k_funcinfo << "NULL cmdFrame" << endl;
 	return;
  }
  cmdFrame()->addUnitDisplayWidget(this);
@@ -92,6 +96,8 @@ public:
 		mUnitDisplayBox = 0;
 
 		mOwner = 0;
+
+		mSelection = 0;
 	}
 
 	QVBoxLayout* mTopLayout;
@@ -108,6 +114,8 @@ public:
 	Player* mOwner;
 
 	QTimer mUpdateTimer;
+
+	BoSelection* mSelection;
 };
 
 BosonCommandFrameBase::BosonCommandFrameBase(QWidget* parent) : QFrame(parent, "cmd frame")
@@ -169,6 +177,16 @@ BosonCommandFrameBase::~BosonCommandFrameBase()
  delete d;
 }
 
+BosonCommandFrameBase* BosonCommandFrameBase::createCommandFrame(QWidget* parent, bool game)
+{
+ if (game) {
+	return new BosonCommandFrame(parent);
+ } else {
+	return new EditorCommandFrame(parent);
+ }
+ return 0;
+}
+
 BosonOrderWidget* BosonCommandFrameBase::selectionWidget() const
 {
  return d->mSelectionWidget;
@@ -195,25 +213,24 @@ void BosonCommandFrameBase::addUnitDisplayWidget(BoUnitDisplayBase* w)
 
 void BosonCommandFrameBase::slotSelectionChanged(BoSelection* selection)
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(220) << k_funcinfo << endl;
  Unit* leader = 0;
+ d->mSelection = selection;
  if (!selection || selection->count() == 0) {
 	clearSelection();
  } else {
 	leader = selection->leader();
 	if (!leader) {
-		boError() << k_funcinfo << "non-empty selection, but NULL leader" << endl;
+		boError(220) << k_funcinfo << "non-empty selection, but NULL leader" << endl;
 		leader = 0;
 		clearSelection();
 	} else if (!leader->owner()) {
-		boError() << k_funcinfo << "group leader has NULL owner" << endl;
+		boError(220) << k_funcinfo << "group leader has NULL owner" << endl;
 		leader = 0;
 		clearSelection();
-	} else {
-		if (leader->isDestroyed()) {
-			boWarning() << k_funcinfo << "group leader is destroyed" << endl;
-			leader = 0;
-		}
+	} else if (leader->isDestroyed()) {
+		boWarning(220) << k_funcinfo << "group leader is destroyed" << endl;
+		leader = 0;
 	}
  }
 
@@ -252,17 +269,12 @@ void BosonCommandFrameBase::setSelectedUnit(Unit* unit)
 	hidePluginWidgets();
 	return;
  }
- Player* owner = unit->owner();
- if (!owner) {
-	boError() << k_funcinfo << "NULL owner" << endl;
-	return;
- }
  if (!unit->speciesTheme()) {
-	boError() << k_funcinfo << "NULL speciestheme" << endl;
+	boError(220) << k_funcinfo << "NULL speciestheme" << endl;
 	return;
  }
 
- if (owner != localPlayer()) {
+ if (!localPlayerIO() || !localPlayerIO()->ownsUnit(unit)) {
 	// hmm.. is this correct? maybe display for plugins anyway? dont know...
 	hidePluginWidgets();
 	selectionWidget()->hideOrderButtons();
@@ -299,83 +311,88 @@ Player* BosonCommandFrameBase::localPlayer() const
  return d->mOwner;
 }
 
-void BosonCommandFrameBase::slotPlaceUnit(ProductionType t, unsigned long int unitType)
+PlayerIO* BosonCommandFrameBase::localPlayerIO() const
 {
  if (!localPlayer()) {
-	boError() << k_funcinfo << "NULL local player" << endl;
+	return 0;
+ }
+ return localPlayer()->playerIO();
+}
+
+void BosonCommandFrameBase::slotPlaceUnit(const BoSpecificAction& action)
+{
+ if (!localPlayerIO()) {
+	boError(220) << k_funcinfo << "NULL local player IO" << endl;
 	return;
  }
- if (t != ProduceUnit) {
-	boError() << k_funcinfo << "Only ProduceUnit supported" << endl;
-	return;
- }
- emit signalPlaceUnit(unitType, localPlayer());
- emit signalAction(ActionBuild); // placement preview
+ emit signalPlaceUnit(action.productionId(), localPlayerIO()->player());
+ BoSpecificAction a;
+ a.setType(ActionPlacementPreview);
+ emit signalAction(a);
 }
 
-void BosonCommandFrameBase::slotPlaceCell(int groundType)
+void BosonCommandFrameBase::slotPlaceGround(unsigned int textureCount, unsigned char* alpha)
 {
- emit signalPlaceCell(groundType);
- emit signalAction(ActionBuild); // placement preview
+ emit signalPlaceGround(textureCount, alpha);
+ BoSpecificAction a;
+ a.setType(ActionPlacementPreview);
+ emit signalAction(a);
 }
 
-void BosonCommandFrameBase::slotProduce(ProductionType type, unsigned long int id)
+void BosonCommandFrameBase::slotProduce(const BoSpecificAction& _action)
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(220) << k_funcinfo << endl;
+ BoSpecificAction action = _action;
  if (selectedUnit()) {
-	if (localPlayer() != selectedUnit()->owner()) {
-		boError() << k_funcinfo << "local owner != selected unit owner" << endl;
+	if (!localPlayerIO() || !localPlayerIO()->ownsUnit(selectedUnit())) {
+		boError(220) << k_funcinfo << "local player doesn't own selected unit" << endl;
 		return;
 	}
-	ProductionPlugin* pp = (ProductionPlugin*)selectedUnit()->plugin(UnitPlugin::Production);
-	if (!pp) {
-		boWarning() << k_funcinfo << "NULL production plugin?!" << endl;
-		return;
-	}
-	if (pp->completedProductionType() == type && pp->completedProductionId() == id) {
-		// the player did not start to place the completed production.
-		// enable the placement preview (aka lock the action)
-		emit signalAction(ActionBuild);
-		return; // do NOT start to produce more here.
+	// FIXME: is there any way not to hardcode this
+	if (action.type() != ActionStopProduceUnit && action.type() != ActionStopProduceTech) {
+		ProductionPlugin* pp = (ProductionPlugin*)selectedUnit()->plugin(UnitPlugin::Production);
+		if (!pp) {
+			boWarning(220) << k_funcinfo << "NULL production plugin?!" << endl;
+			return;
+		}
+		if (pp->completedProductionType() == action.productionType() &&
+				pp->completedProductionId() == action.productionId()) {
+			// the player did not start to place the completed production.
+			// enable the placement preview (aka lock the action)
+			action.setType(ActionPlacementPreview);
+			emit signalAction(action);
+			return; // do NOT start to produce more here.
+		}
 	}
  }
- emit signalProduce(type, id, (UnitBase*)selectedUnit(), localPlayer());
-}
-
-void BosonCommandFrameBase::slotStopProduction(ProductionType type, unsigned long int id)
-{
- if (selectedUnit()) {
-	if (localPlayer() != selectedUnit()->owner()) {
-		boError() << k_funcinfo << "local player != owner of selected unit" << endl;
-		return;
-	}
- }
- emit signalStopProduction(type, id, (UnitBase*)selectedUnit(), localPlayer());
+ boDebug(220) << k_funcinfo << "Emitting signalAction(action)  (for producing; type: " << action.type() << ")" << endl;
+ action.setUnit(selectedUnit());
+ emit signalAction(action);
 }
 
 void BosonCommandFrameBase::slotUpdateProduction(Unit* f)
 {
  if (!f) {
-	boError() << k_funcinfo << "NULL unit" << endl;
+	boError(220) << k_funcinfo << "NULL unit" << endl;
 	return;
  }
- if (selectedUnit() == f && selectionWidget()->orderType() == OrderProduce) {
-	boDebug() << k_funcinfo << endl;
+ if (selectedUnit() == f && selectionWidget()->isProduceAction()) {
+	boDebug(220) << k_funcinfo << endl;
 	setProduction(f);
  }
 }
 
 void BosonCommandFrameBase::slotUpdateProductionOptions()
 {
- if (selectedUnit() && selectionWidget()->orderType() == OrderProduce) {
-	boDebug() << k_funcinfo << endl;
+ if (selectedUnit() && selectionWidget()->isProduceAction()) {
+	boDebug(220) << k_funcinfo << endl;
 	setProduction(selectedUnit());
  }
 }
 
 void BosonCommandFrameBase::slotUpdate()
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(220) << k_funcinfo << endl;
  if (!selectedUnit()) {
 	d->mUpdateTimer.stop();
 	return;
@@ -410,15 +427,10 @@ bool BosonCommandFrameBase::checkUpdateTimer() const
  return use;
 }
 
-void BosonCommandFrameBase::slotSetButtonsPerRow(int b)
-{
- selectionWidget()->setButtonsPerRow(b);
-}
-
 void BosonCommandFrameBase::reparentMiniMap(QWidget* map)
 {
  if (!map) {
-	boError() << k_funcinfo << "NULL map" << endl;
+	boError(220) << k_funcinfo << "NULL map" << endl;
 	return;
  }
  map->reparent(this, QPoint(0,0));
@@ -451,3 +463,8 @@ void BosonCommandFrameBase::addUnitView()
  d->mUnitView->setBackgroundOrigin(WindowOrigin);
 }
 
+BoSelection* BosonCommandFrameBase::selection() const
+{
+ return d->mSelection;
+}
+
