@@ -30,10 +30,20 @@
 #define BO_TILE_SIZE 48
 #define BO_GL_CELL_SIZE 1.0
 
+// both must be > 0.0:
+#define BO_GL_NEAR_PLANE 1.0f // FIXME: should be > 1.0
+#define BO_GL_FAR_PLANE 100.0f
+
 
 #define BOSON_COOKIE 992
 #define BOSON_PORT 5454
 
+
+// magic cookies for binary files
+#define BOSONMAP_MAP_MAGIC_COOKIE QString::fromLatin1("BosonMap")
+#define BOSONMAP_TEXMAP_MAGIC_COOKIE QString::fromLatin1("BoTexMap")
+
+
 #define MAXIMAL_ADVANCE_COUNT 19 // maximal value of advanceCount param emitted by Boson::signalAdvance
 
 #define BOSON_MAX_PLAYERS 10 // test if this is working - 2 is tested
@@ -86,19 +96,22 @@
 #define DEFAULT_CHAT_SCREEN_MAX_ITEMS 5
 #define DEFAULT_MAGNIFICATION_FILTER GL_LINEAR
 #define DEFAULT_MINIFICATION_FILTER GL_LINEAR
-#define DEFAULT_MIPMAP_MINIFICATION_FILTER GL_NEAREST_MIPMAP_NEAREST
+#define DEFAULT_MIPMAP_MINIFICATION_FILTER GL_LINEAR_MIPMAP_NEAREST
 #define DEFAULT_ALIGN_SELECTION_BOXES true
 #define DEFAULT_RMB_MOVES_WITH_ATTACK true
 #define DEFAULT_MOUSE_WHEEL_ACTION CameraZoom
 #define DEFAULT_MOUSE_WHEEL_SHIFT_ACTION CameraRotate
 #define DEFAULT_DEACTIVATE_WEAPON_SOUNDS false
-#define DEFAULT_USE_LIGHT false // not really supported yet
+#define DEFAULT_USE_LIGHT true // not really supported yet
+#define DEFAULT_USE_MATERIALS false
 #define DEFAULT_USE_MIPMAPS_FOR_MODELS true
 #define DEFAULT_TOOLTIP_UPDATE_PERIOD 300
 #define DEFAULT_TOOLTIP_CREATOR 1 // FIXME: should be BoToolTipCreator::Extended, but I don't want to include the file here
+#define DEFAULT_USE_LOD true
 
 // not in BosonConfig (not necessarily in OptionsDialog):
 #define DEFAULT_CHEAT_MODE true // by default cheating is *enabled* (debugging)
+#define DEFAULT_GROUND_RENDERER 0
 
 #endif
 
