@@ -20,11 +20,14 @@
 #include "bosonconfig.h"
 
 #include "bosoncursor.h"
-#include "bosonmodel.h"
+#include "bosonmodeltextures.h"
 #include "bosontexturearray.h"
 #include "boson.h"
 #include "defines.h"
 #include "bodebug.h"
+#include "bogltooltip.h"
+#include "bogroundrenderer.h"
+#include "bo3dtools.h"
 
 #include <klocale.h>
 #include <knuminput.h>
@@ -37,6 +40,8 @@
 #include <qcheckbox.h>
 #include <qvbox.h>
 #include <qtooltip.h>
+#include <qpushbutton.h>
+#include <qlineedit.h>
 
 #include "optionswidgets.moc"
 
@@ -48,6 +53,7 @@ OptionsWidget::OptionsWidget()
 
 OptionsWidget::~OptionsWidget()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 
@@ -73,10 +79,14 @@ GeneralOptions::GeneralOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mCmdBackground->insertItem(i18n("None"));
  //TODO: display filename only... - not the complete path
  mCmdBackground->insertStringList(mCmdBackgrounds);
+
+ mRMBMovesWithAttack = new QCheckBox("Units attack enemies in sight while moving", this);
+ mRMBMovesWithAttack->setChecked(boConfig->RMBMovesWithAttack());
 }
 
 GeneralOptions::~GeneralOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 void GeneralOptions::apply()
@@ -89,12 +99,13 @@ void GeneralOptions::apply()
  if (mGameSpeed->value() != game()->gameSpeed()) {
 	game()->slotSetGameSpeed(mGameSpeed->value());
  }
- emit signalMiniMapScaleChanged(mMiniMapScale->value());
+ boConfig->setMiniMapScale(mMiniMapScale->value());
  QString file;
  if (mCmdBackground->currentItem() > 0) {
 	file = mCmdBackgrounds[mCmdBackground->currentItem() - 1];
  }
  emit signalCmdBackgroundChanged(file);
+ boConfig->setRMBMovesWithAttack(mRMBMovesWithAttack->isChecked());
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -106,6 +117,7 @@ void GeneralOptions::load()
  }
  setGameSpeed(game()->gameSpeed());
  setMiniMapScale(boConfig->miniMapScale());
+ setRMBMovesWithAttack(boConfig->RMBMovesWithAttack());
  // TODO: cmdbackground
 }
 
@@ -114,6 +126,7 @@ void GeneralOptions::setDefaults()
  setGameSpeed(DEFAULT_GAME_SPEED);
  setMiniMapScale(DEFAULT_MINIMAP_SCALE);
  setCmdBackground(QString::null);
+ setRMBMovesWithAttack(DEFAULT_RMB_MOVES_WITH_ATTACK);
 }
 
 void GeneralOptions::setGameSpeed(int ms)
@@ -136,6 +149,11 @@ void GeneralOptions::setCmdBackground(const QString& file)
  }
 }
 
+void GeneralOptions::setRMBMovesWithAttack(bool attack)
+{
+ mRMBMovesWithAttack->setChecked(attack);
+}
+
 
 //////////////////////////////////////////////////////////////////////
 // Cursor Options
@@ -168,6 +186,7 @@ CursorOptions::CursorOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 
 CursorOptions::~CursorOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 void CursorOptions::apply()
@@ -196,7 +215,19 @@ void CursorOptions::setDefaults()
 
 void CursorOptions::load()
 {
-#warning TODO
+ setCursor((CursorMode)boConfig->cursorMode());
+ int dirIndex = -1;
+ if (boConfig->cursorDir().isNull()) {
+	dirIndex = 0;
+ } else {
+	dirIndex = BosonCursor::availableThemes().findIndex(boConfig->cursorDir());
+ }
+ if (dirIndex < 0) {
+	boWarning() << k_funcinfo << "could not find cusor theme " << boConfig->cursorDir() << endl;
+	dirIndex = 0;
+ }
+ mCursorTheme->setCurrentItem(dirIndex);
+ QString dir = boConfig->cursorDir();
 }
 
 void CursorOptions::setCursor(CursorMode mode)
@@ -229,10 +260,31 @@ ScrollingOptions::ScrollingOptions(QWidget* parent) : QVBox(parent), OptionsWidg
  mArrowSpeed->setRange(1, 200);
  mArrowSpeed->setLabel(i18n("Arrow key steps"));
 
+ QMap<int, QString> items;
+ items.insert((int)CameraMove, i18n("Move camera"));
+ items.insert((int)CameraZoom, i18n("Zoom camera"));
+ items.insert((int)CameraRotate, i18n("Rotate camera"));
+ hbox = new QHBox(this);
+ (void)new QLabel(i18n("Mouse wheel action"), hbox);
+ mMouseWheelAction = new QComboBox(hbox);
+
+ hbox = new QHBox(this);
+ (void)new QLabel(i18n("Mouse wheel + shift action"), hbox);
+ mMouseWheelShiftAction = new QComboBox(hbox);
+
+ QMap<int, QString>::Iterator it = items.begin();
+ for (; it != items.end(); ++it) {
+	mMouseWheelAction->insertItem(it.data(), it.key());
+	mMouseWheelShiftAction->insertItem(it.data(), it.key());
+ }
+
+ mMouseWheelAction->setCurrentItem((int)DEFAULT_MOUSE_WHEEL_ACTION);
+ mMouseWheelShiftAction->setCurrentItem((int)DEFAULT_MOUSE_WHEEL_SHIFT_ACTION);
 }
 
 ScrollingOptions::~ScrollingOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 void ScrollingOptions::apply()
@@ -248,6 +300,8 @@ void ScrollingOptions::apply()
 	mArrowSpeed->setValue(0);
  }
  boConfig->setArrowKeyStep(mArrowSpeed->value());
+ boConfig->setMouseWheelAction((CameraAction)(mMouseWheelAction->currentItem()));
+ boConfig->setMouseWheelShiftAction((CameraAction)(mMouseWheelShiftAction->currentItem()));
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -257,6 +311,8 @@ void ScrollingOptions::setDefaults()
  setCursorEdgeSensity(DEFAULT_CURSOR_EDGE_SENSITY);
  setRMBScrolling(DEFAULT_USE_RMB_MOVE);
  setMMBScrolling(DEFAULT_USE_MMB_MOVE);
+ mMouseWheelAction->setCurrentItem((int)DEFAULT_MOUSE_WHEEL_ACTION);
+ mMouseWheelShiftAction->setCurrentItem((int)DEFAULT_MOUSE_WHEEL_SHIFT_ACTION);
 }
 
 void ScrollingOptions::load()
@@ -265,6 +321,8 @@ void ScrollingOptions::load()
  setCursorEdgeSensity(boConfig->cursorEdgeSensity());
  setRMBScrolling(boConfig->rmbMove());
  setMMBScrolling(boConfig->mmbMove());
+ setMouseWheelAction(boConfig->mouseWheelAction());
+ setMouseWheelShiftAction(boConfig->mouseWheelShiftAction());
 }
 
 void ScrollingOptions::setCursorEdgeSensity(int s)
@@ -287,6 +345,16 @@ void ScrollingOptions::setMMBScrolling(bool on)
  mMMBScrolling->setChecked(on);
 }
 
+void ScrollingOptions::setMouseWheelAction(CameraAction action)
+{
+ mMouseWheelAction->setCurrentItem((int)action);
+}
+
+void ScrollingOptions::setMouseWheelShiftAction(CameraAction action)
+{
+ mMouseWheelShiftAction->setCurrentItem((int)action);
+}
+
 
 //////////////////////////////////////////////////////////////////////
 // Sound Options
@@ -296,8 +364,7 @@ SoundOptions::SoundOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 {
  (void)new QLabel(i18n("Disable the following unit sounds (please send a bug report if you can think of more descriptive names):"), this);
  QCheckBox* c;
- c = new QCheckBox(i18n("Shoot"), this);
- mCheckBox2UnitSoundEvent.insert(c, SoundShoot);
+ weaponsounds = new QCheckBox(i18n("Weapon sounds"), this);
  c = new QCheckBox(i18n("Order Move"), this);
  mCheckBox2UnitSoundEvent.insert(c, SoundOrderMove);
  c = new QCheckBox(i18n("Order Attack"), this);
@@ -314,6 +381,7 @@ SoundOptions::SoundOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 
 SoundOptions::~SoundOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 void SoundOptions::apply()
@@ -323,6 +391,7 @@ void SoundOptions::apply()
  for (; it != mCheckBox2UnitSoundEvent.end(); ++it) {
 	boConfig->setUnitSoundActivated(it.data(), !it.key()->isChecked());
  }
+ boConfig->setDeactivateWeaponSounds(weaponsounds->isChecked());
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -332,11 +401,13 @@ void SoundOptions::setDefaults()
  for (; it != mCheckBox2UnitSoundEvent.end(); ++it) {
 	it.key()->setChecked(false);
  }
+ weaponsounds->setChecked(DEFAULT_DEACTIVATE_WEAPON_SOUNDS);
 }
 
 void SoundOptions::load()
 {
  setUnitSoundsDeactivated(boConfig);
+ weaponsounds->setChecked(boConfig->deactivateWeaponSounds());
 }
 
 void SoundOptions::setUnitSoundsDeactivated(BosonConfig* conf)
@@ -357,33 +428,66 @@ void SoundOptions::setUnitSoundsDeactivated(BosonConfig* conf)
 
 OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 {
- // AB: most entries here are very technical (especially the texture filters).
- // put them into a separate widget which can be reached using a "advanced"
- // button. then add some default options (i.e. best speed, average speed, best
- // quality)
- mUpdateInterval = new KIntNumInput(DEFAULT_UPDATE_INTERVAL, this);
+ QHBox* hbox = new QHBox(this);
+
+ if (!boConfig->hasKey("RenderingSpeed")) {
+	boConfig->addDynamicEntry(new BoConfigIntEntry(boConfig, "RenderingSpeed", (int)Defaults));
+ }
+ (void)new QLabel(i18n("Rendering speed/quality"), hbox);
+ mRenderingSpeed = new QComboBox(hbox);
+ mRenderingSpeed->insertItem(i18n("Defaults"));
+ mRenderingSpeed->insertItem(i18n("Highest quality"));
+ mRenderingSpeed->insertItem(i18n("Lowest quality (e.g. for software rendering)"));
+ connect(mRenderingSpeed, SIGNAL(activated(int)), this, SLOT(slotRenderingSpeedChanged(int)));
+
+ mAlignSelectBoxes = new QCheckBox(this);
+ mAlignSelectBoxes->setText(i18n("Align unit selection boxes to camera"));
+
+ QVBox* atiWorkaround = new QVBox(this);
+ mEnableATIDepthWorkaround = new QCheckBox(atiWorkaround);
+ mEnableATIDepthWorkaround->setText(i18n("Enable ATI depth workaround"));
+ connect(mEnableATIDepthWorkaround, SIGNAL(toggled(bool)), this, SLOT(slotEnableATIDepthWorkaround(bool)));
+ QToolTip::add(mEnableATIDepthWorkaround, i18n("Use this if you own a ATI card and you have <em>extreme</em> problems at selecting units"));
+ QHBox* atiValue = new QHBox(atiWorkaround);
+ (void)new QLabel(i18n("Value: "), atiValue);
+ mATIDepthWorkaroundValue = new QLineEdit(atiValue);
+ QPushButton* atiDefaultValue = new QPushButton(i18n("Default"), atiValue);
+ connect(atiDefaultValue, SIGNAL(clicked()), this, SLOT(slotATIDepthWorkaroundDefaultValue()));
+ slotATIDepthWorkaroundDefaultValue();
+ mEnableATIDepthWorkaround->setChecked(false);
+ slotEnableATIDepthWorkaround(false);
+
+ QPushButton* showDetails = new QPushButton(i18n("Show &Details"), this);
+ showDetails->setToggleButton(true);
+ connect(showDetails, SIGNAL(toggled(bool)), this, SLOT(slotShowDetails(bool)));
+
+ mAdvanced = new QVBox(this);
+ showDetails->setOn(false);
+ slotShowDetails(false);
+
+ mUpdateInterval = new KIntNumInput(DEFAULT_UPDATE_INTERVAL, mAdvanced);
  mUpdateInterval->setRange(1, 100);
  mUpdateInterval->setLabel(i18n("Update interval (low values hurt performance)"));
  QToolTip::add(mUpdateInterval, i18n("The update interval specifies after how many milli seconds the scene gets re-rendered and therefore directly influence the frames per seconds. But low values prevent boson from doing other important tasks and therefore you might end up in a game that takes several seconds until your units react to your commands! 20ms are usually a good value."));
- mModelTexturesMipmaps = new QCheckBox(this);
+ mModelTexturesMipmaps = new QCheckBox(mAdvanced);
  mModelTexturesMipmaps->setText(i18n("Use mipmaps for model textures"));
  QToolTip::add(mModelTexturesMipmaps, i18n("With mipmapping disabled units often look ugly - but it consumes less memory and therefore might improve rendering speed."));
 
- QHBox* hbox = new QHBox(this);
+ hbox = new QHBox(mAdvanced);
  (void)new QLabel(i18n("Texture Magnification Filter"), hbox);
  mMagnificationFilter = new QComboBox(hbox);
  mMagnificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
  mMagnificationFilter->insertItem(i18n("Use GL_LINEAR (best quality)"));
  QToolTip::add(mMagnificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a bigger size than they are stored at. This applies to all textures.")); // AB: also note how big the impact on speed is!
 
- hbox = new QHBox(this);
+ hbox = new QHBox(mAdvanced);
  (void)new QLabel(i18n("Texture Minification Filter"), hbox);
  mMinificationFilter = new QComboBox(hbox);
  mMinificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
  mMinificationFilter->insertItem(i18n("Use GL_LINEAR (best quality)"));
  QToolTip::add(mMinificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a smaller size than they are stored at. This applies to not-mipmapped textures only.")); // AB: also note how big the impact on speed is!
 
- hbox = new QHBox(this);
+ hbox = new QHBox(mAdvanced);
  (void)new QLabel(i18n("Texture Mipmap Minification Filter"), hbox);
  mMipmapMinificationFilter = new QComboBox(hbox);
  mMipmapMinificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
@@ -394,20 +498,139 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mMipmapMinificationFilter->insertItem(i18n("Use GL_LINEAR_MIPMAP_LINEAR (best quality)"));
  QToolTip::add(mMipmapMinificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a smaller size than they are stored at. This applies to mipmapped textures (i.e. model textures) only.\nNote: The speed GL_*_MIPMAP_* will probably be noticebly slower, but it's quality is way better!"));
 
- mAlignSelectBoxes = new QCheckBox(this);
- mAlignSelectBoxes->setText(i18n("Align unit selection boxes to camera"));
+ mUseLight = new QCheckBox(mAdvanced);
+ mUseLight->setText(i18n("Enable light"));
+
+ mUseMaterials = new QCheckBox(mAdvanced);
+ mUseMaterials->setText(i18n("Use materials"));
+ QToolTip::add(mUseMaterials, i18n("Materials influence the way in which models (like units) are lighted. You can disable them to gain some performance."));
+
+ hbox = new QHBox(mAdvanced);
+ (void)new QLabel(i18n("Ground rendering method"), hbox);
+ mGroundRenderer = new QComboBox(hbox);
+ for (int i = 0; i < BoGroundRenderer::Last; i++) {
+	mGroundRenderer->insertItem(BoGroundRenderer::rttiToName(i));
+ }
+ mGroundRenderer->setCurrentItem(0);
+
+ mUseLOD = new QCheckBox(i18n("Use level of detail"), mAdvanced);
+ hbox = new QHBox(mAdvanced);
+ (void)new QLabel(i18n("Default level of detail"), hbox);
+ mDefaultLOD = new QComboBox(hbox);
+ mDefaultLOD->insertItem(i18n("All details (default)"));
+ mDefaultLOD->insertItem(i18n("Lowest details"));
+ mDefaultLOD->setCurrentItem(0);
+ connect(mUseLOD, SIGNAL(toggled(bool)), hbox, SLOT(setEnabled(bool)));
 }
 
 OpenGLOptions::~OpenGLOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
+}
+
+void OpenGLOptions::slotRenderingSpeedChanged(int index)
+{
+ setRenderingSpeed(indexToRenderingSpeed(index));
+}
+
+int OpenGLOptions::renderingSpeed() const
+{
+ return indexToRenderingSpeed(mRenderingSpeed->currentItem());
+}
+
+int OpenGLOptions::indexToRenderingSpeed(int index)
+{
+ switch (index) {
+	default:
+		boWarning(210) << k_funcinfo << "invalid index: " << index << endl;
+	case 0:
+		return Defaults;
+	case 1:
+		return BestQuality;
+	case 2:
+		return Fastest;
+ }
+ return BestQuality;
+}
+
+int OpenGLOptions::renderingSpeedToIndex(int speed)
+{
+ switch (speed) {
+	default:
+		boWarning(210) << k_funcinfo << "invalid value: " << speed << endl;
+	case Defaults:
+		return 0;
+	case BestQuality:
+		return 1;
+	case Fastest:
+		return 2;
+ }
+ return 0;
+}
+
+void OpenGLOptions::setRenderingSpeed(int speed)
+{
+ switch ((RenderingSpeed)speed) {
+	default:
+		boWarning(210) << k_funcinfo << "invalid value: " << speed << endl;
+	case Defaults:
+		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
+		mModelTexturesMipmaps->setChecked(DEFAULT_USE_MIPMAPS_FOR_MODELS);
+		setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
+		setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
+		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
+		mUseLight->setChecked(DEFAULT_USE_LIGHT);
+		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
+		mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+		setUseLOD(true);
+		setDefaultLOD(0);
+	case BestQuality:
+		// we mostly use defaults here.
+		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
+		mModelTexturesMipmaps->setChecked(true);
+		setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
+		setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
+		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
+		mUseLight->setChecked(DEFAULT_USE_LIGHT);
+		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
+		mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+		setUseLOD(true);
+		setDefaultLOD(0);
+		break;
+	case Fastest:
+		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
+		mModelTexturesMipmaps->setChecked(false);
+		setMagnificationFilter(GL_NEAREST);
+		setMinificationFilter(GL_NEAREST);
+		setMipmapMinificationFilter(GL_NEAREST);
+		mUseLight->setChecked(false);
+		mUseMaterials->setChecked(false);
+		mGroundRenderer->setCurrentItem(BoGroundRenderer::Fast);
+		setUseLOD(true);
+		setDefaultLOD(5000);
+		break;
+	
+ }
+ mRenderingSpeed->setCurrentItem(renderingSpeedToIndex(speed));
+}
+
+void OpenGLOptions::slotShowDetails(bool show)
+{
+ if (show) {
+	mAdvanced->show();
+ } else {
+	mAdvanced->hide();
+ }
 }
 
 void OpenGLOptions::apply()
 {
  boDebug(210) << k_funcinfo << endl;
+ boConfig->setIntValue("RenderingSpeed", renderingSpeed());
+
  bool reloadModelTextures = false;
  bool resetTexParameter = false;
- emit signalUpdateIntervalChanged((unsigned int)mUpdateInterval->value());
+ boConfig->setUpdateInterval((unsigned int)mUpdateInterval->value());
  if (boConfig->modelTexturesMipmaps() != mModelTexturesMipmaps->isChecked()) {
 	boConfig->setModelTexturesMipmaps(mModelTexturesMipmaps->isChecked());
 	reloadModelTextures = true;
@@ -433,7 +656,7 @@ void OpenGLOptions::apply()
 	reloadModelTextures = (r == KMessageBox::Yes);
 	if (reloadModelTextures) {
 		boDebug(210) << k_funcinfo << "reloading all textures" << endl;
-		BosonModel::reloadAllTextures();
+		BosonModelTextures::modelTextures()->reloadTextures();
 	}
  }
  if (resetTexParameter) {
@@ -446,28 +669,63 @@ void OpenGLOptions::apply()
 	}
  }
 
+ boConfig->setUIntValue("GroundRenderer", mGroundRenderer->currentItem());
+ emit signalGroundRendererChanged(mGroundRenderer->currentItem());
+
  boConfig->setAlignSelectionBoxes(mAlignSelectBoxes->isChecked());
+ boConfig->setUseLight(mUseLight->isChecked());
+ boConfig->setUseMaterials(mUseMaterials->isChecked());
+ boConfig->setUseLOD(useLOD());
+ boConfig->setUIntValue("DefaultLOD", defaultLOD());
+ boConfig->setBoolValue("EnableATIDepthWorkaround", mEnableATIDepthWorkaround->isChecked());
+ if (mEnableATIDepthWorkaround->isChecked()) {
+	bool ok = false;
+	double d = mATIDepthWorkaroundValue->text().toDouble(&ok);
+	if (!ok) {
+		d = 0.00390625;
+	}
+	boConfig->setDoubleValue("ATIDepthWorkaroundValue", d);
+	Bo3dTools::enableReadDepthBufferWorkaround((float)d);
+ } else {
+	Bo3dTools::disableReadDepthBufferWorkaround();
+ }
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
 void OpenGLOptions::setDefaults()
 {
+ mRenderingSpeed->setCurrentItem(0);
+ slotRenderingSpeedChanged(0);
+
  setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
- mModelTexturesMipmaps->setChecked(true);
+ mModelTexturesMipmaps->setChecked(DEFAULT_USE_MIPMAPS_FOR_MODELS);
  setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
  setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
  setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
  setAlignSelectionBoxes(DEFAULT_ALIGN_SELECTION_BOXES);
+ mUseLight->setChecked(DEFAULT_USE_LIGHT);
+ mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
+ mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+ setUseLOD(DEFAULT_USE_LOD);
+ setDefaultLOD(0);
 }
 
 void OpenGLOptions::load()
 {
+ setRenderingSpeed(boConfig->intValue("RenderingSpeed", Defaults));
  setUpdateInterval(boConfig->updateInterval());
  mModelTexturesMipmaps->setChecked(boConfig->modelTexturesMipmaps());
  setMagnificationFilter(boConfig->magnificationFilter());
  setMinificationFilter(boConfig->minificationFilter());
  setMipmapMinificationFilter(boConfig->mipmapMinificationFilter());
  setAlignSelectionBoxes(boConfig->alignSelectionBoxes());
+ mUseLight->setChecked(boConfig->useLight());
+ mUseMaterials->setChecked(boConfig->useMaterials());
+ mGroundRenderer->setCurrentItem(boConfig->uintValue("GroundRenderer"));
+ setUseLOD(boConfig->useLOD());
+ setDefaultLOD(boConfig->uintValue("DefaultLOD", 0));
+ mEnableATIDepthWorkaround->setChecked(boConfig->boolValue("EnableATIDepthWorkaround"));
+ mATIDepthWorkaroundValue->setText(QString::number(boConfig->doubleValue("ATIDepthWorkaroundValue")));
 }
 
 void OpenGLOptions::setUpdateInterval(int ms)
@@ -577,18 +835,64 @@ void OpenGLOptions::setAlignSelectionBoxes(bool align)
  mAlignSelectBoxes->setChecked(align);
 }
 
+bool OpenGLOptions::useLOD() const
+{
+ return mUseLOD->isChecked();
+}
+
+void OpenGLOptions::setUseLOD(bool use)
+{
+ mUseLOD->setChecked(use);
+}
+
+unsigned int OpenGLOptions::defaultLOD() const
+{
+ switch (mDefaultLOD->currentItem()) {
+	case 0:
+		return 0;
+	default:
+	case 1:
+		// just a random, very high, number.
+		// this will result in lowest details possible.
+		return 5000;
+ }
+ return 0;
+}
+
+void OpenGLOptions::setDefaultLOD(unsigned int l)
+{
+ switch (l) {
+	case 0:
+		mDefaultLOD->setCurrentItem(0);
+		break;
+	default:
+		mDefaultLOD->setCurrentItem(1);
+		break;
+ }
+}
+
+void OpenGLOptions::slotEnableATIDepthWorkaround(bool e)
+{
+ mATIDepthWorkaroundValue->setEnabled(e);
+}
+
+void OpenGLOptions::slotATIDepthWorkaroundDefaultValue()
+{
+ mATIDepthWorkaroundValue->setText(QString::number(0.00390625));
+}
+
 //////////////////////////////////////////////////////////////////////
 // Chat Options
 //////////////////////////////////////////////////////////////////////
 
 ChatOptions::ChatOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 {
- QHBox* hbox = new QHBox(this);
+// QHBox* hbox = new QHBox(this);
  mScreenRemoveTime = new KIntNumInput(DEFAULT_CHAT_SCREEN_REMOVE_TIME, this);
  mScreenRemoveTime->setRange(0, 400);
  mScreenRemoveTime->setLabel(i18n("Remove from screen after seconds (0 to remove never)"));
 
- hbox = new QHBox(this);
+// hbox = new QHBox(this);
  mScreenMaxItems = new KIntNumInput(DEFAULT_CHAT_SCREEN_REMOVE_TIME, this);
  mScreenMaxItems->setRange(-1, 40);
  mScreenMaxItems->setLabel(i18n("Maximal items on the screen (-1 is unlimited)"));
@@ -596,6 +900,7 @@ ChatOptions::ChatOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
 
 ChatOptions::~ChatOptions()
 {
+ boDebug(210) << k_funcinfo << endl;
 }
 
 void ChatOptions::apply()
@@ -628,3 +933,74 @@ void ChatOptions::setScreenMaxItems(int m)
  mScreenMaxItems->setValue(m);
 }
 
+//////////////////////////////////////////////////////////////////////
+// ToolTip Options
+//////////////////////////////////////////////////////////////////////
+
+ToolTipOptions::ToolTipOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
+{
+ // you could reduce the update period to very low values to monitor changes
+ // that change very often (you may want to use that in combination with pause),
+ // such as waypoints (?), health, reload state, ...
+ mUpdatePeriod = new KIntNumInput(DEFAULT_TOOLTIP_UPDATE_PERIOD, this);
+ mUpdatePeriod->setRange(1, 500);
+ mUpdatePeriod->setLabel(i18n("Update period. Low values lead to more current data. Debugging option - leave this at the default.)"));
+
+ // TODO: the tooltip delay!
+
+ QHBox* hbox = new QHBox(this);
+ (void)new QLabel(i18n("Tooltip kind:"), hbox);
+ mToolTipCreator = new QComboBox(hbox);
+ BoToolTipCreatorFactory factory;
+ QValueList<int> tips = factory.availableTipCreators();
+ for (unsigned int i = 0; i < tips.count(); i++) {
+	mToolTipCreator->insertItem(factory.tipCreatorName(tips[i]));
+ }
+}
+
+ToolTipOptions::~ToolTipOptions()
+{
+ boDebug(210) << k_funcinfo << endl;
+}
+
+void ToolTipOptions::apply()
+{
+ BoToolTipCreatorFactory factory;
+ QValueList<int> tips = factory.availableTipCreators();
+ int index = mToolTipCreator->currentItem();
+ if (index >= 0 && index < (int)tips.count()) {
+	boConfig->setToolTipCreator(tips[index]);
+ } else {
+	boWarning() << k_funcinfo << "invalid tooltip creator index=" << index << endl;
+ }
+ boConfig->setToolTipUpdatePeriod(mUpdatePeriod->value());
+}
+
+void ToolTipOptions::setDefaults()
+{
+ mUpdatePeriod->setValue(DEFAULT_TOOLTIP_UPDATE_PERIOD);
+ BoToolTipCreatorFactory factory;
+ QValueList<int> tips = factory.availableTipCreators();
+ int index = -1;
+ for (unsigned int i = 0; i < tips.count(); i++) {
+	if (tips[i] == DEFAULT_TOOLTIP_CREATOR) {
+		index = i;
+	}
+ }
+ mToolTipCreator->setCurrentItem(index);
+}
+
+void ToolTipOptions::load()
+{
+ mUpdatePeriod->setValue(boConfig->toolTipUpdatePeriod());
+ BoToolTipCreatorFactory factory;
+ QValueList<int> tips = factory.availableTipCreators();
+ int index = -1;
+ for (unsigned int i = 0; i < tips.count(); i++) {
+	if (tips[i] == boConfig->toolTipCreator()) {
+		index = i;
+	}
+ }
+ mToolTipCreator->setCurrentItem(index);
+}
+
