@@ -19,6 +19,7 @@
 
 #include "player.h"
 
+#include "playerio.h"
 #include "speciestheme.h"
 #include "unit.h"
 #include "unitproperties.h"
@@ -27,10 +28,11 @@
 #include "bosonstatistics.h"
 #include "boson.h"
 #include "upgradeproperties.h"
-#include "unitpropertyhandler.h"
+#include "items/bosonitempropertyhandler.h"
 #include "bosonpropertyxml.h"
 #include "bosonprofiling.h"
 #include "bodebug.h"
+#include "bobincoder.h"
 
 #include <kgame/kgame.h>
 #include <kgame/kgamemessage.h>
@@ -50,6 +52,8 @@ public:
 		mMap = 0;
 
 		mStatistics = 0;
+
+		mPlayerIO = 0;
 	}
 
 	QPtrList<Unit> mUnits;
@@ -65,6 +69,8 @@ public:
 
 	int mMobilesCount;
 	int mFacilitiesCount;
+
+	PlayerIO* mPlayerIO;
 };
 
 Player::Player() : KPlayer()
@@ -88,6 +94,7 @@ Player::Player() : KPlayer()
 		KGamePropertyBase::PolicyLocal, "MineralCost");
  d->mOil.registerData(IdOil, dataHandler(),
 		KGamePropertyBase::PolicyLocal, "OilCost");
+ d->mPlayerIO = new PlayerIO(this);
 
  quitGame(); // this will reset some variables
 }
@@ -98,10 +105,21 @@ Player::~Player()
  quitGame(true);
  dataHandler()->clear(); // this must not be in quitGame()
  delete mSpecies;
+ delete d->mPlayerIO;
  delete d;
  boDebug() << k_funcinfo << "done" << endl;
 }
 
+PlayerIO* Player::playerIO() const
+{
+ return d->mPlayerIO;
+}
+
+BosonMap* Player::map() const
+{
+ return d->mMap;
+}
+
 void Player::quitGame(bool destruct)
 {
  boDebug() << k_funcinfo << endl;
@@ -187,12 +205,14 @@ void Player::loadTheme(const QString& species, const QColor& teamColor)
  mSpecies = new SpeciesTheme(species, teamColor);
 }
 
-void Player::addUnit(Unit* unit)
+void Player::addUnit(Unit* unit, int dataHandlerId)
 {
- d->mUnitPropID++;// used for ID of KGamePropertyHandler
  d->mUnits.append(unit);
- unit->setOwner(this); // already done in c'tor of Unit
- unit->dataHandler()->registerHandler(BosonMessage::UnitPropertyHandler + d->mUnitPropID, this,
+ if (dataHandlerId == -1) {
+	dataHandlerId = BosonMessage::UnitPropertyHandler + d->mUnitPropID;
+	d->mUnitPropID++;// used for ID of KGamePropertyHandler
+ }
+ unit->dataHandler()->registerHandler(dataHandlerId, this,
 		SLOT(sendProperty(int, QDataStream&, bool*)),
 		SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
 
@@ -254,13 +274,16 @@ void Player::slotUnitPropertyChanged(KGamePropertyBase* prop)
  }
 
  // AB: we don't check for sender()->isA() here, for performance reasons.
- UnitPropertyHandler* p = (UnitPropertyHandler*)sender();
- if (!p->unit()) {
-	boError() << k_funcinfo << "NULL parent unit for property handler" << endl;
+ BosonItemPropertyHandler* p = (BosonItemPropertyHandler*)sender();
+ if (!p->item()) {
+	boError() << k_funcinfo << "NULL parent item for property handler" << endl;
 	boDebug() << "player=" << id() << ",propId=" << prop->id() << endl;
 	return;
  }
- Unit* unit = (Unit*)p->unit();
+ if (!RTTI::isUnit(p->item()->rtti())) {
+	return;
+ }
+ Unit* unit = (Unit*)p->item();
  if (!unit) {
 	boError() << k_funcinfo << "NULL unit" << endl;
 	return;
@@ -320,27 +343,6 @@ bool Player::save(QDataStream& stream)
  return true;
 }
 
-bool Player::saveUnits(QDataStream& stream)
-{
- // Save units
- Q_UINT32 unitCount = d->mUnits.count();
- stream << unitCount;
- for (unsigned int i = 0; i < unitCount; i++) {
-	Unit* unit = d->mUnits.at(i);
-	// we need the type first!
-	stream << (unsigned long int)unit->type();
-	stream << (unsigned long int)unit->id();
-
-	stream << (int)unit->dataHandler()->id();
-
-	if (!unit->save(stream)) {
-		boError() << k_funcinfo << "Error while saving unit with id=" << unit->id() << endl;
-		return false;
-	}
- }
- return true;
-}
-
 bool Player::load(QDataStream& stream)
 {
  boDebug() << k_funcinfo << endl;
@@ -381,51 +383,6 @@ bool Player::load(QDataStream& stream)
  return true;
 }
 
-bool Player::loadUnits(QDataStream& stream)
-{
- // Load units
- Q_UINT32 unitCount;
- stream >> unitCount;
- for (unsigned int i = 0; i < unitCount; i++) {
-	unsigned long int type;
-	unsigned long int id;
-	int dataHandlerID;
-	stream >> type;
-	stream >> id;
-	stream >> dataHandlerID;
-
-	// Create unit with Boson
-	Unit* unit = ((Boson*)game())->loadUnit(type, this);
-
-	if (!unit) {
-		boError() << k_funcinfo << "NULL unit loaded" << endl;
-		// we cannot load properly anymore.
-		return false;
-	}
-
-	// Set additional properties
-	d->mUnits.append(unit);
-	unit->dataHandler()->registerHandler(dataHandlerID, this,
-			SLOT(sendProperty(int, QDataStream&, bool*)),
-			SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
-	unit->setId(id);
-
-	// Call unit's loading methods
-	if (!unit->load(stream)) {
-		boError() << k_funcinfo << "Error while loading unit with id=" << id << endl;
-		return false;
-	}
-
-	// Increase unit count
-	if (unit->isMobile()) {
-		d->mMobilesCount++;
-	} else {
-		d->mFacilitiesCount++;
-	}
- }
- return true;
-}
-
 QPtrList<Unit>* Player::allUnits() const
 {
  return &(d->mUnits);
@@ -687,8 +644,8 @@ bool Player::saveAsXML(QDomElement& root)
 {
  BosonProfiler profiler(BosonProfiling::SavePlayerToXML);
 
- // AB: probably we'll never use this... (KGame should take care of the id)
- root.setAttribute(QString::fromLatin1("Id"), id());
+ // note: we need to save the index in the list, not the actual id()
+ root.setAttribute(QString::fromLatin1("Id"), game()->playerList()->findRef(this));
 
  root.setAttribute(QString::fromLatin1("NetworkPriority"), networkPriority());
 
@@ -703,20 +660,6 @@ bool Player::saveAsXML(QDomElement& root)
  }
  root.appendChild(handler);
 
- // save units
- QDomElement units = doc.createElement(QString::fromLatin1("Units"));
- QPtrListIterator<Unit> it(d->mUnits);
- for (; it.current(); ++it) {
-	Unit* u = it.current();
-	QDomElement unit = doc.createElement(QString::fromLatin1("Unit"));
-	if (!u->saveAsXML(unit)) {
-		boError() << k_funcinfo << "Could not save unit " << u->id() << endl;
-		continue;
-	}
-	units.appendChild(unit);
- }
- root.appendChild(units);
-
  // Save speciestheme
  if (speciesTheme()) {
 	root.setAttribute(QString::fromLatin1("SpeciesTheme"), speciesTheme()->identifier());
@@ -746,19 +689,6 @@ bool Player::loadFromXML(const QDomElement& root)
  // this does NOT load the units!
 
  bool ok = false;
- int id = root.attribute(QString::fromLatin1("Id")).toInt(&ok);
- if (!ok) {
-	boError(260) << k_funcinfo << "Id was no valid number" << endl;
-	return false;
- }
- Q_UNUSED(id);
-// setId(id); // AB: KGame should take care of this. we should not need this. (remember that we depend on KGame::d->mUniquePlayerNumber to be in sync!)
- int networkPriority = root.attribute(QString::fromLatin1("NetworkPriority")).toInt(&ok);
- if (!ok) {
-	boError(260) << k_funcinfo << "NetworkPriority was no valid number" << endl;
-	return false;
- }
- setNetworkPriority(networkPriority);
 
  boDebug(260) << k_funcinfo << "load data handler" << endl;
  // load the data handler
@@ -769,50 +699,27 @@ bool Player::loadFromXML(const QDomElement& root)
 	return false;
  }
 
- // note: this does NOT load the units.
-
-
- boDebug(260) << k_funcinfo << "loading speciesTheme" << endl;
- QString speciesIdentifier;
- QColor color;
- if (root.hasAttribute(QString::fromLatin1("SpeciesTheme"))) {
-	speciesIdentifier = root.attribute(QString::fromLatin1("SpeciesTheme"));
- }
- if (root.hasAttribute(QString::fromLatin1("TeamColor"))) {
-	unsigned int c = root.attribute(QString::fromLatin1("TeamColor")).toUInt(&ok);
+ if (root.hasAttribute(QString::fromLatin1("UnitPropId"))) {
+	d->mUnitPropID = root.attribute(QString::fromLatin1("UnitPropId")).toUInt(&ok);
 	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid TeamColor value" << endl;
-	} else {
-		color.setRgb(c);
+		boError(260) << k_funcinfo << "Invalid UnitPropId value" << endl;
+		return false;
 	}
  }
- if (speciesIdentifier.isNull()) {
-	// migh be valid if we ever use this for network loading, too!
-	boError(260) << k_funcinfo << "No SpeciesTheme" << endl;
-	return false;
- } else {
-	boDebug(260) << k_funcinfo << "speciesTheme: " << speciesIdentifier << endl;
-	// TODO: check whether this theme actually exists and could be loaded
-	loadTheme(SpeciesTheme::speciesDirectory(speciesIdentifier), color);
- }
-
- d->mUnitPropID = root.attribute(QString::fromLatin1("UnitPropId")).toUInt(&ok);
- if (!ok) {
-	boError(260) << k_funcinfo << "Invalid UnitPropId value" << endl;
-	return false;
- }
 
 
  // Load fog
- boDebug(260) << k_funcinfo << "loading fow" << endl;
- loadFogOfWar(root);
+ if (!root.namedItem(QString::fromLatin1("Fogged")).isNull()) {
+	boDebug(260) << k_funcinfo << "loading fow" << endl;
+	if (!loadFogOfWar(root)) {
+		boError(260) << k_funcinfo << "loading fog of war failed" << endl;
+		return false;
+	}
+ }
 
  // Load statistics
  QDomElement statistics = root.namedItem(QString::fromLatin1("Statistics")).toElement();
- if (statistics.isNull()) {
-	boWarning() << k_funcinfo << "No Statistics tag" << endl;
-	// dont return
- } else {
+ if (!statistics.isNull()) {
 	d->mStatistics->load(statistics);
  }
 
@@ -820,86 +727,7 @@ bool Player::loadFromXML(const QDomElement& root)
  return true;
 }
 
-bool Player::loadUnitsFromXML(const QDomElement& root)
-{
- if (root.isNull()) {
-	boError(260) << k_funcinfo << "NULL root node" << endl;
-	return false;
- }
- QDomElement units = root.namedItem(QString::fromLatin1("Units")).toElement();
- if (units.isNull()) {
-	boWarning(260) << k_funcinfo << "no units for player " << id() << endl;
-	return true;
- }
- QDomNodeList list = units.elementsByTagName(QString::fromLatin1("Unit"));
- if (list.count() == 0) {
-	boWarning(260) << k_funcinfo << "no units for player " << id() << endl;
-	return true;
- }
- for (unsigned int i = 0; i < list.count(); i++) {
-	QDomElement e = list.item(i).toElement();
-	if (e.isNull()) {
-		boError(260) << k_funcinfo << i << " is not an element" << endl;
-		return false;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("UnitType"))) {
-		boError(260) << k_funcinfo << "missing attribute: UnitType for Unit " << i << endl;
-		continue;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("Id"))) {
-		boError(260) << k_funcinfo << "missing attribute: Id for Unit " << i << endl;
-		continue;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("DataHandlerId"))) {
-		boError(260) << k_funcinfo << "missing attribute: DataHandlerId for Unit " << i << endl;
-		continue;
-	}
-	bool ok = false;
-	unsigned long int type;
-	unsigned long int id;
-	int dataHandlerId;
-	type = e.attribute(QString::fromLatin1("UnitType")).toULong(&ok);
-	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid UnitType number for Unit " << i << endl;
-		continue;
-	}
-	id = e.attribute(QString::fromLatin1("Id")).toULong(&ok);
-	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid Id number for Unit " << i << endl;
-		continue;
-	}
-	dataHandlerId = e.attribute(QString::fromLatin1("DataHandlerId")).toInt(&ok);
-	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid DataHandlerId number for Unit " << i << endl;
-		continue;
-	}
-
-	// Create unit with Boson
-	Unit* unit = ((Boson*)game())->loadUnit(type, this);
-
-	// Set additional properties
-	d->mUnits.append(unit);
-	unit->dataHandler()->registerHandler(dataHandlerId, this,
-			SLOT(sendProperty(int, QDataStream&, bool*)),
-			SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
-	unit->setId(id);
-
-	// Call unit's loading methods
-	if (!unit->loadFromXML(e)) {
-		boWarning(260) << k_funcinfo << "Could not load unit " << id << " correctly" << endl;
-	}
-
-	// Increase unit count
-	if (unit->isMobile()) {
-		d->mMobilesCount++;
-	} else {
-		d->mFacilitiesCount++;
-	}
- }
- return true;
-}
-
-void Player::saveFogOfWar(QDomElement& root) const
+bool Player::saveFogOfWar(QDomElement& root) const
 {
  // AB: I've tried several ways of doing this but couldn't find the solution. so
  // now we simply stream every single bit as a complete byte. not good, but
@@ -908,50 +736,51 @@ void Player::saveFogOfWar(QDomElement& root) const
  // this trouble with strings/xml and e.g. \0
  QDomDocument doc = root.ownerDocument();
  QDomElement fog = doc.createElement(QString::fromLatin1("Fogged"));
- QString text;
- QTextOStream s(&text);
- s << (Q_UINT32)d->mFogged.size();
- s << ' ';
- for (unsigned int i = 0; i < d->mFogged.size(); i++) {
-	s << (char)(d->mFogged.testBit(i) ? '1' : '0');
- }
- QDomCDATASection fow = doc.createCDATASection(text);
- fog.appendChild(fow);
+ fog.appendChild(doc.createTextNode(BoBinCoder::toCoded(d->mFogged)));
  root.appendChild(fog);
+ return true;
 }
 
-void Player::loadFogOfWar(const QDomElement& root)
+bool Player::loadFogOfWar(const QDomElement& root)
 {
  QDomElement element = root.namedItem(QString::fromLatin1("Fogged")).toElement();
  if (element.isNull()) {
 	boError() << k_funcinfo << "No Fogged tag found" << endl;
 	d->mFogged.fill(true);
-	return;
+	return false;
  }
  QString text = element.text();
  if (text.isEmpty()) {
 	boError() << k_funcinfo << "no content for Fogged tag found" << endl;
 	d->mFogged.fill(true);
-	return;
+	return false;
  }
- QTextIStream s(&text);
- s.flags(s.flags() & ~QTextStream::skipws);
- Q_UINT32 size;
- s >> size;
-
- QChar space; // note: QTextStream skips whitespaces for char, not for QChar
- s >> space;
- if (space != ' ') {
-	boError() << k_funcinfo << "oops - format error. read: " << space.latin1() << " should be: " << ' ' << endl;
-	// we can not even resize the flog array, because it may be just any
-	// random value. we can't depend on it being correct
-	return;
- }
- d->mFogged.resize(size);
- for (unsigned int i = 0; i < size; i++) {
-	char bit;
-	s >> bit;
-	d->mFogged.setBit(i, bit == '1');
+ boDebug() << k_funcinfo << "decoding" << endl;
+ d->mFogged = BoBinCoder::toBinary(text);
+ boDebug() << k_funcinfo << "decoded: " << d->mFogged.size() << endl;
+ if (d->mMap) {
+	if (d->mMap->width() * d->mMap->height() != d->mFogged.size()) {
+		boError() << k_funcinfo << "fog of war loaded: " << d->mFogged.size() << " map size: " << d->mMap->width() << "x" << d->mMap->height() << endl;
+		return false;
+	}
  }
+ return true;
+}
+
+void Player::writeGameLog(QTextStream& log)
+{
+ log << "Player: " << id() << endl;
+ log << minerals() << " " << oil() << endl;
+ log << mobilesCount() << " " << facilitiesCount() << endl;
+
+ QPtrListIterator<Unit> it(d->mUnits);
+ Unit* u;
+ while (it.current()) {
+	u = it.current();
+	log << "Unit: " << u->id() << "  " << u->x() << " " << u->y() << " " << u->z() << "  " << u->rotation() <<
+			"  " << u->speed() << "  " << u->health() << "  " << u->work() << " " << u->advanceWork() << endl;
+	++it;
+ }
+ log << endl;
 }
 
