@@ -19,6 +19,7 @@
 
 #include "unit.h"
 #include "player.h"
+#include "playerio.h"
 #include "bosoncanvas.h"
 #include "cell.h"
 #include "speciestheme.h"
@@ -36,6 +37,7 @@
 #include "bosonweapon.h"
 #include "bopointeriterator.h"
 #include "bodebug.h"
+#include "bo3dtools.h"
 
 #include <kgame/kgamepropertylist.h>
 #include <kgame/kgame.h>
@@ -49,6 +51,9 @@
 
 #define MAX_PATH_AGE 5
 
+#define USE_SMOOTH_HEIGHT
+
+
 bool Unit::mInitialized = false;
 
 class Unit::UnitPrivate
@@ -67,6 +72,7 @@ public:
 	KGameProperty<int> mWantedRotation;
 	KGameProperty<int> mMoveAttacking;
 	KGameProperty<int> mSearchPath;
+	KGameProperty<int> mSlowDownAtDestination;
 
 	// be *very* careful with those - NewGameDialog uses Unit::save() which
 	// saves all KGameProperty objects. If non-KGameProperty properties are
@@ -84,8 +90,7 @@ public:
 };
 
 Unit::Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas)
-		: UnitBase(prop),
-		BosonItem(owner->speciesTheme() ? owner->speciesTheme()->unitModel(prop->typeId()) : 0, canvas)
+		: UnitBase(prop, owner, canvas)
 {
  if (!mInitialized) {
 	initStatic();
@@ -94,13 +99,12 @@ Unit::Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas)
  mCurrentPlugin = 0;
  mAdvanceFunction = &Unit::advanceNone;
  mAdvanceFunction2 = &Unit:: advanceNone;
- setOwner(owner);
  d->mPlugins.setAutoDelete(true);
  d->mPlugins.clear();
 
  // note: these width and height can be used for e.g. pathfinding. It does not
  // depend in any way on the .3ds file or another OpenGL thing.
- setSize(prop->unitWidth(), prop->unitHeight());
+ setSize(prop->unitWidth(), prop->unitHeight(), prop->unitDepth());
 
  registerData(&d->mWaypoints, IdWaypoints);
  registerData(&d->mMoveDestX, IdMoveDestX);
@@ -109,12 +113,14 @@ Unit::Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas)
  registerData(&d->mWantedRotation, IdWantedRotation);
  registerData(&d->mMoveAttacking, IdMoveAttacking);
  registerData(&d->mSearchPath, IdSearchPath);
+ registerData(&d->mSlowDownAtDestination, IdSlowDownAtDestination);
  d->mMoveDestX.setLocal(0);
  d->mMoveDestY.setLocal(0);
  d->mMoveRange.setLocal(0);
  d->mWantedRotation.setLocal(0);
  d->mMoveAttacking.setLocal(0);
  d->mSearchPath.setLocal(0);
+ d->mSlowDownAtDestination.setLocal(1);
 
  setAnimated(true);
 
@@ -170,9 +176,9 @@ void Unit::initStatic()
  addPropertyId(IdWantedRotation, QString::fromLatin1("WantedRotation"));
  addPropertyId(IdMoveAttacking, QString::fromLatin1("MoveAttacking"));
  addPropertyId(IdSearchPath, QString::fromLatin1("SearchPath"));
+ addPropertyId(IdSlowDownAtDestination, QString::fromLatin1("SlowDownAtDestination"));
 
  // MobileUnit
- addPropertyId(IdSpeed, QString::fromLatin1("Speed"));
  addPropertyId(IdMovingFailed, QString::fromLatin1("MovingFailed"));
  addPropertyId(IdPathRecalculated, QString::fromLatin1("PathRecalculated"));
  addPropertyId(IdPathAge, QString::fromLatin1("PathAge"));
@@ -186,6 +192,9 @@ void Unit::initStatic()
  addPropertyId(IdResourcesX, QString::fromLatin1("ResourcesX"));
  addPropertyId(IdResourcesY, QString::fromLatin1("ResourcesY"));
  addPropertyId(IdHarvestingType, QString::fromLatin1("HarvestingType"));
+ addPropertyId(IdBombingPosX, QString::fromLatin1("BombingPosX"));
+ addPropertyId(IdBombingPosY, QString::fromLatin1("BombingPosY"));
+ addPropertyId(IdMinePlacingCounter, QString::fromLatin1("MinePlacingCounter"));
 
  mInitialized = true;
 }
@@ -298,7 +307,13 @@ void Unit::updateSelectBox()
 void Unit::moveBy(float moveX, float moveY, float moveZ)
 {
 // time critical function
- if (!moveX && !moveY && !moveZ || isDestroyed()) {
+ if (!moveX && !moveY && !moveZ) {
+	return;
+ }
+
+ if (isDestroyed()) {
+	// Just move the unit and return. No need to update z or minimap
+	BosonItem::moveBy(moveX, moveY, moveZ);
 	return;
  }
 
@@ -330,7 +345,8 @@ void Unit::updateZ(float moveByX, float moveByY, float* moveByZ, float* rotateX,
  *rotateX = 0.0f;
  *rotateY = 0.0f;
 
- float* heightMap = canvas()->heightMap();
+#ifndef USE_SMOOTH_HEIGHT
+ const float* heightMap = canvas()->heightMap();
  BO_CHECK_NULL_RET(heightMap);
  int heightMapWidth = canvas()->mapWidth() + 1;
 
@@ -383,12 +399,12 @@ void Unit::updateZ(float moveByX, float moveByY, float* moveByZ, float* rotateX,
 		}
 	}
  }
+#else
+ float newZ = canvas()->heightAtPoint(x() + moveByX + width() / 2, y() + moveByY + height() / 2);
+#endif
 
  if (isFlying()) {
-	if (newZ < z() + *moveByZ) {
-		// don't touch moveByZ.
-		return;
-	}
+	newZ += 2.0f;  // Flying units are always 2 units above the ground
  }
  *moveByZ = newZ - z();
 }
@@ -397,7 +413,7 @@ void Unit::advance(unsigned int advanceCount)
 { // time critical function !!!
 // Mostly animation:
  BosonItem::advance(advanceCount);
- 
+
  if (isDestroyed()) {
 	return;
  }
@@ -409,7 +425,7 @@ void Unit::advance(unsigned int advanceCount)
 	}
  }
 
-// Reload shields
+ // Reload shields
  if (shields() < unitProperties()->shields()) {
 	reloadShields(); // AB: maybe we make that method inline one day.
  }
@@ -447,6 +463,10 @@ bool Unit::attackEnemyUnitsInRange()
  for (; *wit; ++wit) {
 	BosonWeapon* w = *wit;
 
+	if (!w->properties()->autoUse()) {
+		continue;
+	}
+
 	if (!w->reloaded()) {
 		continue;
 	}
@@ -464,10 +484,10 @@ bool Unit::attackEnemyUnitsInRange()
 
 	// If unit is mobile, rotate to face the target if it isn't facing it yet
 	if (isMobile()) {
-		float rot = rotationToPoint(target()->x() - x(), target()->y() - y());
+		float rot = Bo3dTools::rotationToPoint(target()->x() - x(), target()->y() - y());
 		if (rot < rotation() - 5 || rot > rotation() + 5) {
 			// Rotate to face target
-			if (QABS(rotation() - rot) > (2 * speed())) {
+			if (QABS(rotation() - rot) > unitProperties()->rotationSpeed()) {
 				turnTo((int)rot);
 				setAdvanceWork(WorkTurn);
 				return true;
@@ -611,9 +631,9 @@ void Unit::advanceAttack(unsigned int advanceCount)
  }
 
  if (isMobile()) {
-	float rot = rotationToPoint(target()->x() - x(), target()->y() - y());
+	float rot = Bo3dTools::rotationToPoint(target()->x() - x(), target()->y() - y());
 	if(rot < rotation() - 5 || rot > rotation() + 5) {
-		if(QABS(rotation() - rot) > (2 * speed())) {
+		if(QABS(rotation() - rot) > unitProperties()->rotationSpeed()) {
 			turnTo((int)rot);
 			setAdvanceWork(WorkTurn);
 			return;
@@ -629,7 +649,7 @@ void Unit::advanceAttack(unsigned int advanceCount)
  BosonWeapon* w;
  for (; *wit; ++wit) {
 	w = *wit;
-	if (w->reloaded() && w->canShootAt(target()) && inRange(w->properties()->range(), target())) {
+	if (w->properties()->autoUse() && w->reloaded() && w->canShootAt(target()) && inRange(w->properties()->range(), target())) {
 		shootAt(w, target());
 		if (target()->isDestroyed()) {
 			boDebug(300) << "    " << k_funcinfo << "target destroyed, returning" << endl;
@@ -644,9 +664,13 @@ void Unit::advanceAttack(unsigned int advanceCount)
 void Unit::advanceDestroyed(unsigned int advanceCount)
 {
  // note: the unit/wreckage will get deleted pretty soon
- if (advanceCount % 20 != 0) {
+ if (advanceCount % 10 != 0) {
 	return;
  }
+ if (isVisible()) {
+	// Make unit slowly sink into ground
+	setVelocity(0, 0, -(depth() / (REMOVE_WRECKAGES_TIME * MAXIMAL_ADVANCE_COUNT)) * 1.2);
+ }
 }
 
 void Unit::advancePlugin(unsigned int advanceCount)
@@ -675,14 +699,13 @@ void Unit::advanceTurn(unsigned int)
 	turncw = !turncw;
  }
 
- // TODO: rotating speed should be configurable
- if (a <= (int)(2 * speed())) {
+ if (a <= unitProperties()->rotationSpeed()) {
 	dir = d->mWantedRotation;
  } else {
 	if (turncw) {
-		dir += (int)(2 * speed());
+		dir += unitProperties()->rotationSpeed();
 	} else {
-		dir -= (int)(2 * speed());
+		dir -= unitProperties()->rotationSpeed();
 	}
  }
  // Check for overflows
@@ -733,9 +756,19 @@ void Unit::moveTo(const QPoint& pos, bool attack)
  }
 }
 
-bool Unit::moveTo(float x, float y, int range, bool attack)
+bool Unit::moveTo(float x, float y, int range, bool attack, bool slowDownAtDest)
 {
-
+ // If we're moving already, we don't want to set our speed to 0
+ // Set slowDownAtDestination temporarily to false to achieve that.
+ // Maybe also when advanceWork() == WorkMove ?
+ boDebug() << k_funcinfo << "work(): " << work() << "; advanceWork(): " << advanceWork() << endl;
+ if (!ownerIO()) {
+	BO_NULL_ERROR(ownerIO());
+	return false;
+ }
+ if (work() == WorkMove) {
+	d->mSlowDownAtDestination = 0;
+ }
  stopMoving();
 
  if (range == -1) {
@@ -743,25 +776,22 @@ bool Unit::moveTo(float x, float y, int range, bool attack)
  }
  int cellX = (int)(x / BO_TILE_SIZE);
  int cellY = (int)(y / BO_TILE_SIZE);
- if (!canvas()->cell(cellX, cellY)) {
+ Cell* cell = canvas()->cell(cellX, cellY);
+ if (!cell) {
 	boError() << k_funcinfo << x << "," << y << " is no valid cell!" << endl;
 	return false;
  }
- if (!owner()->isFogged(cellX, cellY)) {
-	Cell* c = canvas()->cell(cellX, cellY);
-	if (!c) {
-		boError() << k_funcinfo << "unit " << id() << ": NULL cell at " << x << "," << y << endl;
-		return false;
-	}
-	// No pathfinding if goal not reachable or occupied and we can see it
-	if (!c->canGo(unitProperties())) {
+ if (ownerIO()->canSee(cell)) {
+	if (!ownerIO()->canGo(this, cell)) {
+		// No pathfinding if goal not reachable or occupied and we can see it
 		boDebug() << k_funcinfo << "unit " << id() << ": Can't go to " << x << "," << y << endl;
 		return false;
 	}
  }
 
- d->mMoveDestX = (int)x;
- d->mMoveDestY = (int)y;
+ // Center of the destination cell
+ d->mMoveDestX = (int)(cellX * BO_TILE_SIZE + width() / 2);
+ d->mMoveDestY = (int)(cellY * BO_TILE_SIZE + height() / 2);
  d->mMoveRange = range;
 
  // Do not find path here!!! It would break pathfinding for groups. Instead, we
@@ -774,6 +804,11 @@ bool Unit::moveTo(float x, float y, int range, bool attack)
  } else {
 	d->mMoveAttacking = 0;
  }
+ if (slowDownAtDest) {
+	d->mSlowDownAtDestination = 1;
+ } else {
+	d->mSlowDownAtDestination = 0;
+ }
 
  return true;
 }
@@ -806,13 +841,11 @@ void Unit::newPath()
  for (int unsigned i = 0; i < path.count(); i++) {
 	addWaypoint(path[i]);
  }
- if ((currentWaypoint().x() == x() + width() / 2) && (currentWaypoint().y() == y() + height() / 2))
- {
+ if ((currentWaypoint().x() == x() + width() / 2) && (currentWaypoint().y() == y() + height() / 2)) {
 	boDebug() << k_funcinfo << "!!!!! First waypoint is unit's current pos! Removing" << endl;
 	waypointDone();
  }
- if (waypointCount() == 0)
- {
+ if (waypointCount() == 0) {
 	// Pathfinder now adds -1; -1 itself, so this shouldn't be reached
 	boError() << k_funcinfo << "!!!!! No valid points in path !!!!! Fix pathfinder !!!!!" << endl;
 	addWaypoint(QPoint(-1, -1));
@@ -886,83 +919,51 @@ bool Unit::saveAsXML(QDomElement& root)
  }
  root.setAttribute(QString::fromLatin1("Target"), (unsigned int)targetId);
 
+ if (d->mPlugins.count() != 0) {
+	QDomDocument doc = root.ownerDocument();
+	QDomElement pluginElement = doc.createElement(QString::fromLatin1("UnitPlugin"));
+	QPtrListIterator<UnitPlugin> it(d->mPlugins);
+	for (; it.current(); ++it) {
+		it.current()->saveAsXML(pluginElement);
+
+		// we won't add the element at all, if it wasn't used.
+		if (pluginElement.hasAttributes() || pluginElement.hasChildNodes()) {
+			pluginElement.setAttribute(QString::fromLatin1("Type"), it.current()->pluginType());
+
+			// AB: atm we support a single instance only anyway.
+			pluginElement.setAttribute(QString::fromLatin1("Instance"), 0);
+			root.appendChild(pluginElement);
+
+			// create an element for the next plugin
+			pluginElement = doc.createElement(QString::fromLatin1("UnitPlugin"));
+		}
+	}
+ }
+
+
  // TODO: somehow save active particle systems
 
  return true;
 }
 
-bool Unit::save(QDataStream& stream)
-{
- //we should probably add pure virtual methods save() and load() to the plugins,
- //in order to store non-KGameProperty data there, too
- // note that UnitBase::save() also saves KGameProperty data of plugins and
- // weapons
- if (!UnitBase::save(stream)) {
-	boError() << "Unit not saved properly" << endl;
-	return false;
- }
- stream << (float)x();
- stream << (float)y();
- stream << (float)z();
-
- // store the current plugin:
- int pluginIndex = 0;
- if (currentPlugin()) {
-	pluginIndex = d->mPlugins.findRef(currentPlugin());
- }
- stream << (Q_INT32)pluginIndex;
-
- // also store the target:
- unsigned long int targetId = 0;
- if (target()) {
-	targetId = target()->id();
- }
- stream << (Q_UINT32)targetId;
-
- return true;
-}
-
 bool Unit::loadFromXML(const QDomElement& root)
 {
  if (!UnitBase::loadFromXML(root)) {
 	boError(260) << k_funcinfo << "Unit not loaded properly" << endl;
 	return false;
  }
- bool ok;
- float x;
- float y;
- float z;
- float rotation;
- x = root.attribute(QString::fromLatin1("x")).toFloat(&ok);
+ bool ok = false;
+ float rotation = root.attribute(QString::fromLatin1("Rotation")).toFloat(&ok);
  if (!ok) {
-	boError() << k_funcinfo << "Invalid value for x tag" << endl;
-	x = 0;
-	// don't stop (will be broken, but unit won't get deleted)
- }
- y = root.attribute(QString::fromLatin1("y")).toFloat(&ok);
- if (!ok) {
-	boError() << k_funcinfo << "Invalid value for y tag" << endl;
-	y = 0;
-	// don't stop (will be broken, but unit won't get deleted)
- }
- z = root.attribute(QString::fromLatin1("z")).toFloat(&ok);
- if (!ok) {
-	boError() << k_funcinfo << "Invalid value for z tag" << endl;
-	z = 0;
-	// don't stop (will be broken, but unit won't get deleted)
- }
- rotation = root.attribute(QString::fromLatin1("Rotation")).toFloat(&ok);
- if (!ok) {
-	boError() << k_funcinfo << "Invalid value for Rotation tag" << endl;
-	rotation = 0;
-	// don't stop (will be broken, but unit won't get deleted)
+	boError(260) << k_funcinfo << "Invalid value for Rotation tag" << endl;
+	rotation = 0.0f;
  }
 
  int pluginIndex = 0;
  if (root.hasAttribute(QString::fromLatin1("CurrentPlugin"))) {
 	pluginIndex = root.attribute(QString::fromLatin1("CurrentPlugin")).toInt(&ok);
 	if (!ok) {
-		boError() << k_funcinfo << "Invalid value for CurrentPlugin tag" << endl;
+		boError(260) << k_funcinfo << "Invalid value for CurrentPlugin tag" << endl;
 		pluginIndex = 0;
 	}
 	if ((unsigned int)pluginIndex >= d->mPlugins.count()) {
@@ -979,7 +980,7 @@ bool Unit::loadFromXML(const QDomElement& root)
  if (root.hasAttribute(QString::fromLatin1("Target"))) {
 	targetId = root.attribute(QString::fromLatin1("Target")).toUInt(&ok);
 	if (!ok) {
-		boError() << k_funcinfo << "Invalid value for Target tag" << endl;
+		boError(260) << k_funcinfo << "Invalid value for Target tag" << endl;
 		targetId = 0;
 	}
  }
@@ -992,60 +993,44 @@ bool Unit::loadFromXML(const QDomElement& root)
 	}
  }
 
- move(x, y, z);
+ if (d->mPlugins.count() != 0) {
+	QDomNodeList list = root.elementsByTagName(QString::fromLatin1("UnitPlugin"));
+	for (unsigned int i = 0; i < list.count(); i++) {
+		QDomElement e = list.item(i).toElement();
+		if (e.isNull()) {
+			continue;
+		}
+		unsigned int type;
+		unsigned int instance;
+		bool ok = false;
+		type = e.attribute(QString::fromLatin1("Type")).toUInt(&ok);
+		if (!ok) {
+			boError() << k_funcinfo << "Type of UnitPlugin " << i << " is not a valid number" << endl;
+			return false;
+		}
+		instance = e.attribute(QString::fromLatin1("Instance")).toUInt(&ok);
+		if (!ok) {
+			boError() << k_funcinfo << "Instance of UnitPlugin " << i << " is not a valid number" << endl;
+			return false;
+		}
+		if (instance != 0) {
+			boError() << k_funcinfo << "instance != 0 for UnitPlugin " << i << " is not yet supported." << endl;
+			return false;
+		}
+		UnitPlugin* p = plugin(type);
+		Q_UNUSED(instance);
+		if (!p) {
+			boWarning() << k_funcinfo << "UnitPlugin " << type << " not found for unit " << id() << endl;
+			continue;
+		}
+		p->loadFromXML(e);
+	}
+ }
  setRotation(rotation);
  setAdvanceWork(advanceWork());
  return true;
 }
 
-bool Unit::load(QDataStream& stream)
-{
- if (!UnitBase::load(stream)) {
-	boError() << k_funcinfo << "Unit not loaded properly" << endl;
-	return false;
- }
- //AB: the frame number was loaded/saved as well. i removed this because we
- //shouldn't have this in loading code. frame is dependant on the current
- //actions/animations/... but not relevant to actual game code. it is just the
- //visible appearance
- float x;
- float y;
- float z;
- Q_INT32 pluginIndex;
- Q_UINT32 targetId;
-
- stream >> x;
- stream >> y;
- stream >> z;
- stream >> pluginIndex;
- stream >> targetId;
-
- if (pluginIndex <= 0) {
-	mCurrentPlugin = 0;
- } else {
-	if ((unsigned int)pluginIndex >= d->mPlugins.count()) {
-		boWarning() << k_funcinfo << "Invalid current plugin index: " << pluginIndex << endl;
-		mCurrentPlugin = 0;
-	} else {
-		mCurrentPlugin = d->mPlugins.at(pluginIndex);
-	}
- }
-
- // note: don't use setTarget() here, as it does some additional calculations
- if (targetId == 0) {
-	d->mTarget = 0;
- } else {
-	d->mTarget = boGame->findUnit(targetId, 0);
-	if (!d->mTarget) {
-		boWarning() << k_funcinfo << "Could not find target with unitId=" << targetId << endl;
-	}
- }
-
- move(x, y, z);
- setAdvanceWork(advanceWork());
- return true;
-}
-
 bool Unit::inRange(unsigned long int r, Unit* target) const
 {
  return (QMAX(QABS((target->x() - x()) / BO_TILE_SIZE), QABS((target->y() - y()) / BO_TILE_SIZE)) <= (float)r);
@@ -1067,7 +1052,7 @@ void Unit::shootAt(BosonWeapon* w, Unit* target)
  }
 // boDebug() << id() << " shoots at unit " << target->id() << endl;
  w->shoot(target);
- owner()->statistics()->increaseShots();
+ ownerIO()->statistics()->increaseShots();
 }
 
 BoItemList* Unit::unitsInRange(unsigned long int range) const
@@ -1082,10 +1067,13 @@ BoItemList* Unit::unitsInRange(unsigned long int range) const
  // collisions() does this anyway.
  QRect rect;
  rect.setCoords(left - range, top - range, right + range, bottom + range);
+
+ // TODO: we should do this using PlayerIO. It should return items that are
+ // actually visible to us only!
  BoItemList* items = collisions()->collisionsAtCells(rect, (BosonItem*)this, false);
  items->remove((BosonItem*)this);
 
- BoItemList* inRange = new BoItemList();
+ BoItemList* units = new BoItemList();
  BoItemList::Iterator it = items->begin();
  Unit* u;
  for (; it != items->end(); ++it) {
@@ -1096,14 +1084,17 @@ BoItemList* Unit::unitsInRange(unsigned long int range) const
 	if (u->isDestroyed()) {
 		continue;
 	}
-	if (owner()->isFogged((int)u->x() / BO_TILE_SIZE, (int)u->y() / BO_TILE_SIZE)) {
+	if (!ownerIO()->canSee(u)) {
+		continue;
+	}
+	if (!inRange(range, u)) {
 		continue;
 	}
 	// TODO: remove the items from inRange which are not actually in range (hint:
 	// pythagoras)
-	inRange->append(*it);
+	units->append(*it);
  }
- return inRange;
+ return units;
 }
 
 BoItemList* Unit::enemyUnitsInRange(unsigned long int range) const
@@ -1114,7 +1105,7 @@ BoItemList* Unit::enemyUnitsInRange(unsigned long int range) const
  BoItemList::Iterator it = units->begin();
  for (; it != units->end(); ++it) {
 	u = (Unit*)*it;
-	if (owner()->isEnemy(u->owner())) {
+	if (ownerIO()->isEnemyUnit(u)) {
 		enemy->append(u);
 	}
  }
@@ -1235,18 +1226,24 @@ void Unit::playSound(UnitSoundEvent event)
 
 void Unit::turnTo(int deg)
 {
- boDebug() << k_funcinfo << id() << ": turning to " << deg << endl;
+// boDebug() << k_funcinfo << id() << ": turning to " << deg << endl;
  d->mWantedRotation = deg;
 }
 
-QPtrList<BosonParticleSystem>* Unit::particleSystems() const
+const QPtrList<BosonParticleSystem>* Unit::particleSystems() const
 {
  return &(d->mParticleSystems);
 }
 
-void Unit::setParticleSystems(QPtrList<BosonParticleSystem> list)
+void Unit::clearParticleSystems()
+{
+ d->mParticleSystems.clear();
+}
+
+void Unit::setParticleSystems(const QPtrList<BosonParticleSystem>& list)
 {
  d->mParticleSystems = list;
+ canvas()->addParticleSystems(d->mParticleSystems);
 }
 
 void Unit::loadWeapons()
@@ -1258,10 +1255,24 @@ void Unit::loadWeapons()
 
  // since we use an array and not a list we need to count the weapons first.
  int count = 0;
+ bool hasbomb = false;
+ bool hasmine = false;
  QPtrListIterator<PluginProperties> it(*(unitProperties()->plugins()));
  for (; it.current(); ++it) {
 	if (it.current()->pluginType() == PluginProperties::Weapon) {
 		count++;
+		// Check for bombing weapons and create bombing plugin if necessary
+		if (!hasbomb && ((BosonWeaponProperties*)it.current())->shotType() == BosonShot::Bomb) {
+#warning FIXME: this does _not_ belog to plugins, but to weapons!
+			d->mPlugins.append(new BombingPlugin(this));
+			hasbomb = true;
+		}
+		// Check for mine weapons and create mining plugin if necessary
+		if (!hasmine && ((BosonWeaponProperties*)it.current())->shotType() == BosonShot::Mine) {
+#warning FIXME: this does _not_ belog to plugins, but to weapons!
+			d->mPlugins.append(new MiningPlugin(this));
+			hasmine = true;
+		}
 	}
  }
  if (count > MAX_WEAPONS_PER_UNIT) {
@@ -1299,6 +1310,11 @@ int Unit::moveAttacking() const
  return d->mMoveAttacking;
 }
 
+int Unit::slowDownAtDestination() const
+{
+ return d->mSlowDownAtDestination;
+}
+
 int Unit::distance(const Unit* u) const
 {
  // !!! This method returns square of actual distance. You may want to use sqrt() !!!
@@ -1319,7 +1335,7 @@ int Unit::distance(const BoVector3& pos) const
 
 const QColor* Unit::teamColor() const
 {
- return &owner()->teamColor();
+ return &ownerIO()->teamColor();
 }
 
 int Unit::searchPath() const
@@ -1332,6 +1348,17 @@ void Unit::setSearchPath(int search)
  d->mSearchPath = search;
 }
 
+BosonWeapon* Unit::weapon(unsigned long int id) const
+{
+ BoPointerIterator<BosonWeapon> it(d->mWeapons);
+ for (; *it; ++it) {
+	if ((*it)->properties()->id() == id) {
+		return *it;
+	}
+ }
+ return 0;
+}
+
 
 /////////////////////////////////////////////////
 // MobileUnit
@@ -1344,7 +1371,6 @@ public:
 	{
 	}
 
-	KGameProperty<float> mSpeed;
 	KGameProperty<unsigned int> mMovingFailed;
 	KGameProperty<unsigned int> mPathRecalculated;
 	// Maybe it would be better to move this to Unit? Currently we have to reset
@@ -1360,12 +1386,10 @@ MobileUnit::MobileUnit(const UnitProperties* prop, Player* owner, BosonCanvas* c
 {
  d = new MobileUnitPrivate;
 
- registerData(&d->mSpeed, IdSpeed);
  registerData(&d->mMovingFailed, IdMovingFailed);
  registerData(&d->mPathRecalculated, IdPathRecalculated);
  registerData(&d->mPathAge, IdPathAge);
 
- d->mSpeed.setLocal(0);
  d->mMovingFailed.setLocal(0);
  d->mPathRecalculated.setLocal(0);
  d->mPathAge.setLocal(0);
@@ -1380,9 +1404,11 @@ MobileUnit::MobileUnit(const UnitProperties* prop, Player* owner, BosonCanvas* c
  ((Boson*)owner->game())->slotUpdateProductionOptions();
 
  setParticleSystems(unitProperties()->newConstructedParticleSystems(x() + width() / 2, y() + height() / 2, z()));
- canvas->addParticleSystems(*particleSystems());
 
  setRotation((float)(owner->game()->random()->getLong(359)));
+ setMaxSpeed(prop->speed());
+ setAccelerationSpeed(prop->accelerationSpeed());
+ setDecelerationSpeed(prop->decelerationSpeed());
 }
 
 MobileUnit::~MobileUnit()
@@ -1392,8 +1418,9 @@ MobileUnit::~MobileUnit()
 
 void MobileUnit::advanceMoveInternal(unsigned int advanceCount) // this actually needs to be called for every advanceCount.
 {
- if (speed() == 0) {
-	boWarning(401) << k_funcinfo << "unit " << id() << ": speed == 0" << endl;
+ if (maxSpeed() == 0) {
+	// If unit's max speed is 0, it cannot move
+	boError(401) << k_funcinfo << "unit " << id() << ": maxspeed == 0" << endl;
 	stopMoving();
 	return;
  }
@@ -1402,15 +1429,11 @@ void MobileUnit::advanceMoveInternal(unsigned int advanceCount) // this actually
 	// If we're moving with attacking, first check for any enemies in the range
 	//  This prevents units from moving on when you order them to move somewhere
 	//  but there are still enemies in the range.
-	if (moveAttacking() && attackEnemyUnitsInRange()) {
-		boDebug(401) << k_funcinfo << "unit " << id() << ": Enemy units found in range, attacking" << endl;
-		setVelocity(0.0, 0.0, 0.0);  // To prevent moving
-		setMoving(false);
+	if (attackEnemyUnitsInRangeWhileMoving()) {
 		return;
 	}
+	// If there aren't any enemies, find new path
 	newPath();
-	d->mPathAge = 0;
-	setSearchPath(0);
  }
 
  if (waypointCount() == 0) {
@@ -1421,8 +1444,12 @@ void MobileUnit::advanceMoveInternal(unsigned int advanceCount) // this actually
 	//  is wrong here
 	boError(401) << k_funcinfo << "unit " << id() << ": No waypoints" << endl;
 	newPath();
-	d->mPathAge = 0;
-	return;
+	if (waypointCount() == 0) {
+		// Serious problem somewhere
+		boError(401) << k_funcinfo << "unit " << id() << ": Still no waypoints. Aborting" << endl;
+		stopMoving();
+		return;
+	}
  }
 
  boDebug(401) << k_funcinfo << "unit " << id() << endl;
@@ -1443,53 +1470,33 @@ void MobileUnit::advanceMoveInternal(unsigned int advanceCount) // this actually
 		}
 		// TODO: make sure that target() hasn't moved!
 		// if it has moved also adjust waypoints
-		// RL: I'm not sure if it's needed because path will be recalced every time
-		//  new cell is reached anyway
 	} else if (work() == WorkAttack && !target()) {
-		boWarning() << k_funcinfo << id() << " is in WorkAttack, but has NULL target!" << endl;
+		boError() << k_funcinfo << id() << " is in WorkAttack, but has NULL target!" << endl;
+		stopAttacking();
+		return;
 	}
  } else if (moveAttacking()) {
 	// Attack any enemy units in range
 	// Don't check for enemies every time (if we don't have a target) because it
 	//  slows things down
-  // TODO: Maybe check for enemies every time waypoint is reached instead?
 	if (target()) {
-		if (attackEnemyUnitsInRange()) {
-			boDebug(401) << k_funcinfo << "unit " << id() << ": Enemy units found in range, attacking" << endl;
-			setVelocity(0.0, 0.0, 0.0);  // To prevent moving
-			setMoving(false);
+		if (attackEnemyUnitsInRangeWhileMoving()) {
 			return;
 		}
 	}
  }
 
- QPoint wp = currentWaypoint(); // where we go to
- // If both waypoint's coordinates are -1, then it means that either path to
- //  destination can't be found or that we're already at the goal point. Either
- //  way, we stop moving
- if ((wp.x() == -1) && (wp.y() == -1)) {
-	stopMoving();
-	if (work() == WorkNone) {
-		boDebug() << k_funcinfo << id() << ": stopping moving. Turning" << endl;
-		// Turn a bit
-		int turn = (int)rotation() + (owner()->game()->random()->getLong(90) - 45);
-		// Check for overflows
-		if (turn < 0) {
-			turn += 360;
-		} else if (turn > 360) {
-			turn -= 360;
-		}
-		Unit::turnTo(turn);
-		setWork(WorkTurn);
-	}
+ // Get current waypoint and check if it's valid
+ QPoint wp = currentWaypoint();
+ if (checkWaypoint(wp)) {
 	return;
  }
 
+ // x and y are center of the unit here
  int x = (int)(BosonItem::x() + width() / 2);
  int y = (int)(BosonItem::y() + height() / 2);
-
- float xspeed = 0;
- float yspeed = 0;
+ /*boDebug(401) << k_funcinfo << "pos: (" << x << "; " << y << ")" <<
+    "; wp: (" << wp.x() << "; " << wp.y() << "); wpc: " << waypointCount() << endl;*/
 
  // First check if we're at waypoint
  if ((x == wp.x()) && (y == wp.y())) {
@@ -1501,65 +1508,68 @@ void MobileUnit::advanceMoveInternal(unsigned int advanceCount) // this actually
  // This is a bad hack to fix #55926
  if (d->mWayPointReached) {
 	// Check for enemy units in range every time waypoint is reached
-	if (moveAttacking() && attackEnemyUnitsInRange()) {
-		boDebug(401) << k_funcinfo << "unit " << id() << ": Enemy units found in range, attacking" << endl;
-		setVelocity(0.0, 0.0, 0.0);  // To prevent moving
-		setMoving(false);
+	if (attackEnemyUnitsInRangeWhileMoving()) {
 		return;
 	}
-
 	// Path is recalculated every MAX_PATH_AGE waypoints
 	d->mPathAge = d->mPathAge + 1;
 	if (d->mPathAge >= MAX_PATH_AGE || waypointCount() == 0) {
 		boDebug(401) << k_funcinfo << "Searching new path (update)" << endl;
 		newPath();
-		d->mPathAge = 0;
 	}
 
 	wp = currentWaypoint();
-	// FIXME: code duplication. But it must be done here to prevent trying to go
-	//  to -1; -1 coordinates and ending up with wrong rotation
-	if ((wp.x() == -1) && (wp.y() == -1)) {
-		stopMoving();
-		if (work() == WorkNone) {
-			boDebug() << k_funcinfo << id() << ": stopping moving. Turning" << endl;
-			// Turn a bit
-			int turn = (int)rotation() + (owner()->game()->random()->getLong(90) - 45);
-			// Check for overflows
-			if (turn < 0) {
-				turn += 360;
-			} else if (turn > 360) {
-				turn -= 360;
-			}
-			Unit::turnTo(turn);
-			setWork(WorkTurn);
-		}
+	if (checkWaypoint(wp)) {
 		return;
 	}
 	d->mWayPointReached = false;
  }
 
+ // If we're close to destination, decelerate, otherwise accelerate
+ // TODO: we should also slow down when turning at waypoint.
+ if (slowDownAtDestination() && QMAX(QABS(x - destinationX()), QABS(y - destinationY())) <= decelerationDistance()) {
+/*	boDebug(401) << "MOVING: " << "decelerating; pos: (" << x << ", " << y << "); dest: (" <<
+			destinationX() << ", " << destinationY() << "); dist: " <<
+			QMAX(QABS(x - destinationX()), QABS(y - destinationY())) << "; decel. dist: " <<
+			decelerationDistance() << ";  speed: " << speed() << ";  acc/dec/max speed: " << accelerationSpeed() <<
+			"/" << decelerationSpeed() << "/" << maxSpeed() << endl;*/
+	decelerate();
+ } else {
+	accelerate();
+ }
+ /*boDebug() << k_funcinfo << "Speed is now " << speed() << "; exact pos: (" <<
+    BosonItem::x() << "; " << BosonItem::y() << ")" << endl;*/
+
  // Try to go to same x and y coordinates as waypoint's coordinates
  // First x coordinate
  // Slow down if there is less than speed() pixels to go
- if (QABS(wp.x() - x) < speed()) {
-	xspeed = wp.x() - x;
+ // FIXME: don't slow down if direction doesn't change and moving continues
+ float xspeed = 0;
+ float yspeed = 0;
+ float dist = (int)speed();
+ if (dist < 1) {
+	// speed() was 0.something. Make dist 1
+	dist = 1;
+ }
+ float xdiff, ydiff;
+
+ xdiff = wp.x() - x;
+ ydiff = wp.y() - y;
+
+ if (QABS(xdiff) < dist) {
+	xspeed += xdiff;
  } else {
-	xspeed = speed();
-	if (wp.x() < x) {
-		xspeed = -xspeed;
-	}
+	xspeed += (xdiff > 0) ? dist : -dist;
  }
  // Same with y coordinate
- if (QABS(wp.y() - y) < speed()) {
-	yspeed = wp.y() - y;
+ if (QABS(ydiff) < dist) {
+	yspeed += ydiff;
  } else {
-	yspeed = speed();
-	if (wp.y() < y) {
-		yspeed = -yspeed;
-	}
+	yspeed += (ydiff > 0) ? dist : -dist;
  }
 
+
+ //boDebug() << k_funcinfo << "Setting velocity to (" << xspeed << "; " << yspeed << ")" << endl;
  // Set velocity for actual moving
  setVelocity(xspeed, yspeed, 0.0);
  setMoving(true);
@@ -1642,6 +1652,7 @@ void MobileUnit::advanceMoveCheck()
 
 	d->mMovingFailed = d->mMovingFailed + 1;
 	setVelocity(0.0, 0.0, 0.0);
+	setSpeed(0);
 	// If we haven't yet recalculated path, consider unit to be moving
 	setMoving(d->mPathRecalculated == 0);
 
@@ -1671,16 +1682,6 @@ void MobileUnit::advanceMoveCheck()
  //boDebug(401) << k_funcinfo << "unit " << id() << ": done" << endl;
 }
 
-void MobileUnit::setSpeed(float speed)
-{
- d->mSpeed = speed;
-}
-
-float MobileUnit::speed() const
-{
- return d->mSpeed;
-}
-
 void MobileUnit::turnTo(Direction direction)
 {
  if (isDestroyed()) {
@@ -1700,7 +1701,7 @@ void MobileUnit::turnTo()
  float yspeed = yVelocity();
  // Set correct frame
  if ((xspeed == 0) && (yspeed < 0)) { // North
- 	turnTo(North);
+	turnTo(North);
  } else if ((xspeed > 0) && (yspeed < 0)) { // NE
 	turnTo(NorthEast);
  } else if ((xspeed > 0) && (yspeed == 0)) { // East
@@ -1778,14 +1779,16 @@ bool MobileUnit::loadFromXML(const QDomElement& root)
 	boError() << k_funcinfo << "Unit not loaded properly" << endl;
 	return false;
  }
- return true;
-}
 
-bool MobileUnit::load(QDataStream& stream)
-{
- if (!Unit::load(stream)) {
-	boError() << k_funcinfo << "Unit not loaded properly" << endl;
-	return false;
+ bool ok = false;
+ float speed = 0.0f;
+ if (root.hasAttribute("Speed")) {
+	speed = root.attribute("Speed").toFloat(&ok);
+	if (!ok) {
+		boWarning() << k_funcinfo << "Invalid value for Speed attribute" << endl;
+	} else {
+		setSpeed(speed);
+	}
  }
 
  return true;
@@ -1797,15 +1800,8 @@ bool MobileUnit::saveAsXML(QDomElement& root)
 	boError() << k_funcinfo << "Unit not saved properly" << endl;
 	return false;
  }
- return true;
-}
 
-bool MobileUnit::save(QDataStream& stream)
-{
- if (!Unit::save(stream)) {
-	boError() << k_funcinfo << "Unit not saved properly" << endl;
-	return false;
- }
+ root.setAttribute("Speed", speed());
 
  return true;
 }
@@ -1817,6 +1813,53 @@ void MobileUnit::stopMoving()
  d->mMovingFailed = 0;
  d->mPathRecalculated = 0;
  d->mWayPointReached = false;
+ if (slowDownAtDestination()) {
+	setSpeed(0);
+ }
+}
+
+bool MobileUnit::attackEnemyUnitsInRangeWhileMoving()
+{
+ if (moveAttacking() && attackEnemyUnitsInRange()) {
+	boDebug(401) << k_funcinfo << "unit " << id() << ": Enemy units found in range, attacking" << endl;
+	setVelocity(0.0, 0.0, 0.0);  // To prevent moving
+	setSpeed(0);
+	setMoving(false);
+	return true;
+ }
+ return false;
+}
+
+void MobileUnit::newPath()
+{
+ Unit::newPath();
+ d->mPathAge = 0;
+ setSearchPath(0);
+}
+
+bool MobileUnit::checkWaypoint(const QPoint& wp)
+{
+ // If both waypoint's coordinates are -1, then it means that either path to
+ //  destination can't be found or that we're already at the goal point. Either
+ //  way, we stop moving
+ if ((wp.x() == -1) && (wp.y() == -1)) {
+	stopMoving();
+	if (work() == WorkNone) {
+		boDebug() << k_funcinfo << id() << ": stopping moving. Turning" << endl;
+		// Turn a bit
+		int turn = (int)rotation() + (owner()->game()->random()->getLong(90) - 45);
+		// Check for overflows
+		if (turn < 0) {
+			turn += 360;
+		} else if (turn > 360) {
+			turn -= 360;
+		}
+		Unit::turnTo(turn);
+		setWork(WorkTurn);
+	}
+	return true;
+ }
+ return false;
 }
 
 
@@ -1910,10 +1953,6 @@ void Facility::moveTo(float x, float y, int range)
 
 void Facility::setConstructionStep(unsigned int step)
 {
- if (isDestroyed()) {
-	boError() << k_funcinfo << "unit is already destroyed" << endl;
-	return;
- }
  if (step > constructionSteps()) {
 	step = constructionSteps();
  }
@@ -1934,7 +1973,6 @@ void Facility::setConstructionStep(unsigned int step)
 	((Boson*)owner()->game())->slotUpdateProductionOptions();
 	setAnimationMode(UnitAnimationIdle);
 	setParticleSystems(unitProperties()->newConstructedParticleSystems(x() + width() / 2, y() + height() / 2, z()));
-	canvas()->addParticleSystems(*particleSystems());
  }
 }
 
@@ -1949,18 +1987,20 @@ bool Facility::loadFromXML(const QDomElement& root)
 	boError() << k_funcinfo << "Unit not loaded properly" << endl;
 	return false;
  }
- setConstructionStep(d->mConstructionStep);
- return true;
-}
 
-bool Facility::load(QDataStream& stream)
-{
- if (!Unit::load(stream)) {
-	boError() << k_funcinfo << "Unit not loaded properly" << endl;
-	return false;
+ unsigned int modelStep = 0;
+ if (d->mConstructionStep == constructionSteps()) {
+	modelStep = model()->constructionSteps(); // completed construction
+ } else {
+	modelStep = model()->constructionSteps() * d->mConstructionStep / constructionSteps();
  }
+ setGLConstructionStep(modelStep);
 
- setConstructionStep(d->mConstructionStep);
+ // FIXME: remove. this is from Facility::setConstructionStep. we _need_ to load
+ // particles in loadFromXML() - then these lines are obsolete.
+ if (d->mConstructionStep == constructionSteps()) {
+	setParticleSystems(unitProperties()->newConstructedParticleSystems(x() + width() / 2, y() + height() / 2, z()));
+ }
 
  return true;
 }
@@ -1974,12 +2014,3 @@ bool Facility::saveAsXML(QDomElement& root)
  return true;
 }
 
-bool Facility::save(QDataStream& stream)
-{
- if (!Unit::save(stream)) {
-	boError() << "Unit not loaded properly" << endl;
-	return false;
- }
- return true;
-}
-
