@@ -37,6 +37,8 @@
 
 #include <qpair.h>
 #include <qpoint.h>
+#include <qdom.h>
+#include <qstringlist.h>
 
 UnitPlugin::UnitPlugin(Unit* unit)
 {
@@ -291,6 +293,68 @@ void ProductionPlugin::advance(unsigned int)
  }
 }
 
+bool ProductionPlugin::saveAsXML(QDomElement& root) const
+{
+ if (mProductions.count() == 0) {
+	return true;
+ }
+
+ // AB: we don't use a KGamePropertyList here, cause it is a bit more difficult
+ // with the QPair I guess. simply saving this by hand.
+ QStringList list;
+ for (unsigned int i = 0; i < mProductions.count(); i++) {
+	unsigned int type = (unsigned int)(mProductions[i].first);
+	unsigned int id = mProductions[i].second;
+	QString s = QString::fromLatin1("%1 %2").arg(type).arg(id);
+	list.append(s);
+ }
+ QString productionString = list.join(QString::fromLatin1(","));
+ QDomDocument doc = root.ownerDocument();
+ QDomElement e = doc.createElement("ProductionList");
+ root.appendChild(e);
+ e.appendChild(doc.createTextNode(productionString));
+ return true;
+}
+
+bool ProductionPlugin::loadFromXML(const QDomElement& root)
+{
+ mProductions.clear();
+ QDomElement e = root.namedItem("ProductionList").toElement();
+ if (e.isNull()) {
+	// nothing to load here
+	return true;
+ }
+ QString s = e.text();
+ if (s.isEmpty()) {
+	// list is here, but empty
+	return true;
+ }
+ QStringList list = QStringList::split(QString::fromLatin1(","), s);
+ for (unsigned int i = 0; i < list.count(); i++) {
+	QStringList l = QStringList::split(" ", list[i]);
+	if (l.count() != 2) {
+		boWarning() << k_funcinfo << "type and id of a production item must be space separated" << endl;
+		return false;
+	}
+	bool ok = false;
+	unsigned int type = l[0].toUInt(&ok);
+	if (!ok) {
+		boError() << k_funcinfo << "type of production item " << i << " is not a valid number" << endl;
+		return false;
+	}
+	unsigned int id = l[1].toUInt(&ok);
+	if (!ok) {
+		boError() << k_funcinfo << "id of production item " << i << " is not a valid number" << endl;
+		return false;
+	}
+	QPair<ProductionType, unsigned long int> pair;
+	pair.first = (ProductionType)type;
+	pair.second = id;
+	mProductions.append(pair);
+ }
+ return true;
+}
+
 bool ProductionPlugin::contains(ProductionType type, unsigned long int id)
 {
  QPair<ProductionType, unsigned long int> pair;
@@ -358,6 +422,19 @@ void RepairPlugin::repairInRange()
  }*/
 }
 
+bool RepairPlugin::saveAsXML(QDomElement& root) const
+{
+ Q_UNUSED(root);
+ return true;
+}
+
+bool RepairPlugin::loadFromXML(const QDomElement& root)
+{
+ Q_UNUSED(root);
+ return true;
+}
+
+
 HarvesterPlugin::HarvesterPlugin(Unit* unit)
 		: UnitPlugin(unit)
 {
@@ -390,6 +467,31 @@ void HarvesterPlugin::advance(unsigned int)
  }
 }
 
+bool HarvesterPlugin::saveAsXML(QDomElement& root) const
+{
+ unsigned int refineryId = 0;
+ if (mRefinery) {
+	refineryId = mRefinery->id();
+ }
+ root.setAttribute(QString::fromLatin1("Refinery"), refineryId);
+ return true;
+}
+
+bool HarvesterPlugin::loadFromXML(const QDomElement& root)
+{
+ unsigned int refineryId = 0;
+ bool ok = false;
+ refineryId = root.attribute(QString::fromLatin1("Refinery")).toUInt(&ok);
+ if (!ok) {
+	boError() << k_funcinfo << "Invalid number for Refinery attribute" << endl;
+	return false;
+ }
+#warning FIXME
+// mRefinery = ...;
+ return true;
+}
+
+
 void HarvesterPlugin::advanceMine()
 {
  const HarvesterProperties* prop = (HarvesterProperties*)unit()->properties(PluginProperties::Harvester);
@@ -629,6 +731,8 @@ unsigned int HarvesterPlugin::unloadingSpeed() const
 
 BombingPlugin::BombingPlugin(Unit* owner) : UnitPlugin(owner)
 {
+ owner->registerData(&mPosX, Unit::IdBombingPosX);
+ owner->registerData(&mPosY, Unit::IdBombingPosY);
  mWeapon = 0;
  mPosX = 0;
  mPosY = 0;
@@ -739,10 +843,34 @@ void BombingPlugin::advance(unsigned int)
  }
 }
 
+bool BombingPlugin::saveAsXML(QDomElement& root) const
+{
+ unsigned int weaponId = 0;
+ // TODO
+ root.setAttribute(QString::fromLatin1("Weapon"), weaponId);
+ return true;
+}
+
+bool BombingPlugin::loadFromXML(const QDomElement& root)
+{
+ unsigned int weaponId = 0;
+ bool ok = false;
+ weaponId = root.attribute(QString::fromLatin1("Weapon")).toUInt(&ok);
+ if (!ok) {
+	boError() << k_funcinfo << "Invalid number for Weapon attribute" << endl;
+	return false;
+ }
+ // TODO
+ // mWeapon = ...;
+ return true;
+}
+
+
 
 MiningPlugin::MiningPlugin(Unit* owner) : UnitPlugin(owner)
 {
  mWeapon = 0;
+ owner->registerData(&mPlacingCounter, Unit::IdMinePlacingCounter);
  mPlacingCounter = 0;
 }
 
@@ -786,7 +914,7 @@ void MiningPlugin::advance(unsigned int)
 	// Don't place the mine immediately
 	if (mPlacingCounter > 0) {
 		boDebug() << k_funcinfo << "mPlacingCounter: " << mPlacingCounter << endl;
-		mPlacingCounter--;
+		mPlacingCounter = mPlacingCounter - 1;
 		return;
 	}
 
@@ -853,3 +981,26 @@ void MiningPlugin::advance(unsigned int)
  }
 }
 
+bool MiningPlugin::saveAsXML(QDomElement& root) const
+{
+ unsigned int weaponId = 0;
+ // TODO
+ root.setAttribute(QString::fromLatin1("Weapon"), weaponId);
+ return true;
+}
+
+bool MiningPlugin::loadFromXML(const QDomElement& root)
+{
+ unsigned int weaponId = 0;
+ bool ok = false;
+ weaponId = root.attribute(QString::fromLatin1("Weapon")).toUInt(&ok);
+ if (!ok) {
+	boError() << k_funcinfo << "Invalid number for Weapon attribute" << endl;
+	return false;
+ }
+ // TODO
+ // mWeapon = ...;
+ return true;
+}
+
+
